whenever SQLERROR continue;

DROP TABLE PURCHASE_BY_PRODUCT;
DROP TABLE PURCHASE_BY_PGRUPPA;
DROP TABLE PURCHASE_BY_GRUPPA;

CREATE TABLE PURCHASE_BY_PRODUCT
(
  T_DATE,      
  PRODUCT,     
  DEPT,        
  ACTUAL_SUM,  
  PURCHASE_SUM,
  PROFIT_SUM,  
  QTY,         
  CONSTRAINT PURCHASE_BY_PRODUCT_PK 
    PRIMARY KEY (T_DATE, PRODUCT, DEPT)   
)
ORGANIZATION INDEX COMPRESS 2
NOLOGGING
STORAGE(INITIAL 2M NEXT 2M)
AS
SELECT 
  sales.T_DATE,
  sales.PRODUCT,
  sales.DEPT,
  cast(sales.ACTUAL_SUM as number(19,2)),
  cast(sales.PURCHASE_SUM as number(19,2)),
  cast(sales.PROFIT_SUM as number(19,2)),
  cast(sales.QTY as number(19,3))
  FROM (SELECT /*+ FULL(TR), INDEX(L DB1_LINE_PK) */
               decode(tr.debt_account,hextoraw('0000008300DC0013'),tr.crdt_a1,tr.debt_a1) as DEPT,
               decode(tr.debt_account,hextoraw('0000008300DC0013'),tr.crdt_a2,tr.debt_a2) as PRODUCT,
               TRUNC(TR.trans_DATE) as T_DATE, 
               SUM(decode(tr.debt_account,hextoraw('0000008300DC0013'),
                          tr.amount*l.F15007748,
                         -tr.amount*l.F15007748)) as ACTUAL_SUM, 
               SUM(decode(tr.debt_account,hextoraw('0000008300DC0013'),
                          tr.amount*l.F15007745,
                         -tr.amount*l.F15007745)) as PURCHASE_SUM,
               SUM(decode(tr.debt_account,hextoraw('0000008300DC0013'),
                          tr.amount*(l.F15007748-l.F15007745),
                         -tr.amount*(l.F15007748-l.F15007745))) as PROFIT_SUM,
               SUM(decode(tr.debt_account,hextoraw('0000008300DC0013'),
                          tr.amount,-tr.amount)) as QTY 
          FROM DB1_TRANSACTION TR, DB1_LINE L
         WHERE ((TR.DEBT_ACCOUNT=hextoraw('0000008300DC0013'))
            OR (TR.CRDT_ACCOUNT=hextoraw('0000008300DC0013')))
           AND (TR.CURRENCY=hextoraw('0002002401AC0001'))
           and (decode(tr.debt_account,hextoraw('0000008300DC0013'),tr.crdt_a2,tr.debt_a2) is not null)
           AND (l.id=tr.line)
      GROUP BY TRUNC(TR.TRANS_DATE),
               decode(tr.debt_account,hextoraw('0000008300DC0013'),tr.crdt_a1,tr.debt_a1),
               decode(tr.debt_account,hextoraw('0000008300DC0013'),tr.crdt_a2,tr.debt_a2)) sales;

CREATE INDEX PURCHASE_BY_PRODUCT_BY_PRODUCT ON PURCHASE_BY_PRODUCT
(
  PRODUCT,
  DEPT,
  T_DATE)
COMPRESS 2;

exec DBMS_STATS.GATHER_TABLE_STATS('&1','PURCHASE_BY_PRODUCT',NULL,DBMS_STATS.AUTO_SAMPLE_SIZE,FALSE,'FOR ALL COLUMNS',NULL,'AUTO',TRUE);

CREATE TABLE PURCHASE_BY_PGRUPPA
(
  T_DATE,      
  PGRUPPA,     
  DEPT,        
  ACTUAL_SUM,  
  PURCHASE_SUM,
  PROFIT_SUM,  
  QTY,         
  CONSTRAINT PURCHASE_BY_PGRUPPA_PK 
    PRIMARY KEY (T_DATE, PGRUPPA, DEPT)   
)
ORGANIZATION INDEX COMPRESS 2
NOLOGGING
STORAGE(INITIAL 2M NEXT 2M)
AS
SELECT  /*+ INDEX(T DB1_PRODUCT_BY_GROUP) */
      PT.T_DATE,
      T.LOCAL_GROUP,
      PT.DEPT,
      cast(SUM(PT.ACTUAL_SUM) as number(19,2)),  
      cast(SUM(PT.PURCHASE_SUM) as number(19,2)),
      cast(SUM(PT.PROFIT_SUM) as number(19,2)),  
      cast(SUM(PT.QTY) as number(19,3))
  from PURCHASE_BY_PRODUCT PT, DB1_PRODUCT T
 where (T.ID=PT.PRODUCT)
   and (t.local_group is not null)
 GROUP BY PT.T_DATE, T.LOCAL_GROUP, PT.DEPT;

exec DBMS_STATS.GATHER_TABLE_STATS('&1','PURCHASE_BY_PGRUPPA',NULL,DBMS_STATS.AUTO_SAMPLE_SIZE,FALSE,'FOR ALL COLUMNS',NULL,'AUTO',TRUE);

CREATE TABLE PURCHASE_BY_GRUPPA
(
  T_DATE,      
  GRUPPA,     
  DEPT,        
  ACTUAL_SUM,  
  PURCHASE_SUM,
  PROFIT_SUM,  
  QTY,         
  CONSTRAINT PURCHASE_BY_GRUPPA_PK 
    PRIMARY KEY (T_DATE, GRUPPA, DEPT)   
)
ORGANIZATION INDEX COMPRESS 2
NOLOGGING
STORAGE(INITIAL 2M NEXT 2M)
AS
SELECT 
      PT.T_DATE,
      G.PID,
      PT.DEPT,
      cast(SUM(PT.ACTUAL_SUM) as number(19,2)),  
      cast(SUM(PT.PURCHASE_SUM) as number(19,2)),
      cast(SUM(PT.PROFIT_SUM) as number(19,2)),  
      cast(SUM(PT.QTY) as number(19,3))
  from PURCHASE_BY_PGRUPPA PT, DB1_CLASSIF G
 where (G.ID=PT.PGRUPPA)
 GROUP BY PT.T_DATE, G.PID, PT.DEPT;

commit;

exec DBMS_STATS.GATHER_TABLE_STATS('&1','PURCHASE_BY_GRUPPA',NULL,DBMS_STATS.AUTO_SAMPLE_SIZE,FALSE,'FOR ALL COLUMNS',NULL,'AUTO',TRUE);

exit;