-------------------------------------------------------------------------------
--
--         
--
--    
-------------------------------------------------------------------------------


create or replace package pickup as 

  valVersion     varchar2(4)   := '2.8';

  PickupService  DOMINO_USER.ID%TYPE := HEXTORAW('87F2000000000002');

  valNoID        RAW(8) := HEXTORAW('07D2000307D20001'); --  ""
  valYesID       RAW(8) := HEXTORAW('07D2000307D20002'); --  ""

  FUNCTION MetaMessage(error_code in varchar2) return varchar2;

  FUNCTION GetVersion return varchar2;
  FUNCTION GetRegion(RegionCode in varchar2) return raw;
  FUNCTION GetStore(StoreCode in varchar2) return raw;
  FUNCTION GetPartner(PartnerCode in varchar2) return raw;
  FUNCTION GetClient(ClientCode in varchar2) return raw;
  FUNCTION GetProduct(ProductCode in varchar2) return raw;
  FUNCTION GetPartProduct(ProductCode in varchar2) return raw;
  FUNCTION GetBaseProduct(PartProductID in raw) return raw;

  FUNCTION DocumentTypeDescription(DocType in number) return varchar2;
  FUNCTION NDSDescription(NDS in raw) return varchar2;

  FUNCTION FindDocument(DocCode in varchar2, DocType in number) return raw;

  pragma RESTRICT_REFERENCES (GetRegion, WNDS, WNPS, RNPS); 
  pragma RESTRICT_REFERENCES (GetStore, WNDS, WNPS, RNPS); 
  pragma RESTRICT_REFERENCES (GetClient, WNDS, WNPS, RNPS); 


  FUNCTION  PrintCatalog     (ClassifID in raw) return varchar2;

  PROCEDURE Products         (DocID in raw);
  PROCEDURE Product          (ProductID in raw);
  PROCEDURE Barcodes         (ProductID in raw);

  PROCEDURE GetChilds        (DocID in raw);
  PROCEDURE GetDocument      (DocID in raw);
  PROCEDURE GetOrder         (DocID in raw);

  PROCEDURE writelog         (command in varchar2, xmldoc in clob);
  PROCEDURE ChangeOrderState (OrderID in raw, NewState in raw);


  --   --
  PROCEDURE about;                                               --  
--  PROCEDURE list_img_orders (xmldoc in clob);                  --   -      
  PROCEDURE get_img_orders (xmldoc in clob);                     --   -      
  PROCEDURE get_img_order (xmldoc in clob);                      --     
  PROCEDURE create_move_doc (xmldoc in clob);                    --      
  PROCEDURE create_sale_doc (xmldoc in clob);                    --     
  PROCEDURE create_claim_doc (xmldoc in clob);                   --   
  PROCEDURE revoke_pickup (xmldoc in clob);                      --   

END pickup;

--    (   ,      )

/
SHOW ERRORS

-------------------------------------------------------------------------------------------


create or replace package body pickup as


FUNCTION GetVersion return varchar2 is
begin
  return valVersion;
end;


PROCEDURE about IS
begin
  OWA_UTIL.mime_header('text/xml');  
  HTP.print('<?xml version="1.0"?>');
  HTP.print('<package name="DOMINO8.PICKUP" version="' || valVersion ||'" />');
end;


PROCEDURE writelog(command in varchar2, xmldoc in clob) is
begin
  insert into pickuplog (qdate, qtext, xmldoc)  values (sysdate, command, xmldoc);
  commit;
end;


PROCEDURE ChangeOrderState (OrderID in raw, NewState in raw) is
begin
   if ( DOMINO.GetUserID() is null ) then DOMINO.login(PickupService); end if;
   --   
   update DB1_DOCUMENT set F14352406 = NewState where id = OrderID;
   --  
   insert into ZAKAZ_STATE (ZAKAZ, STATE, CHANGE_DATE, USERID) values (OrderID, NewState, SYSDATE, PickupService);
exception when OTHERS then
   null;
end;



FUNCTION MetaMessage(error_code in varchar2) return varchar2 is
error_text varchar2(1000);
begin
   case error_code
      when '0'   then error_text:= null;
      when '400' then error_text:= SQLERRM;
      when '401' then error_text:='   order.';
      when '402' then error_text:='   code.';
      when '403' then error_text:='   date.';
      when '404' then error_text:='   department.';
      when '405' then error_text:='   partner.';
      when '406' then error_text:='   .';
      when '407' then error_text:='  .';
      when '301' then error_text:='     .';
      when '302' then error_text:='     .';
      when '303' then error_text:='  .';
      when '304' then error_text:='     .';
      when '305' then error_text:='     .';
      when '306' then error_text:='     .';
      when '307' then error_text:='   .';
                 else error_text:='  .';
   end case;

   return '<meta return_code="'|| error_code ||'" description="' || DBMS_XMLGEN.convert(error_text) || '"/>';
end;


--       
PROCEDURE Products(DocID in raw) is
begin
   HTP.print('<products>');  
   for c1 in (
       select PRODUCT as Product from DB1_LINE where DOCUMENT = DocID)
   loop
       Product(c1.Product);
   end loop;
   HTP.print('</products>');
end Products;


--   
PROCEDURE Product(ProductID in raw) is
begin
   for c1 in (
       select 
          TPRODUCT.CODE        as ProductCode,
          TPRODUCT.NAME        as ProductName,
          TPRODUCT.ARTICLE     as ProductArticle,
          TPRODUCT.LOCAL_GROUP as ProductGroup,
          TPRODUCT.F14745613   as ProductNDS,
          TPRODUCT.F42401852   as SourceProduct,
          TSUPPLIER.CODE       as ProductSupplier,
          TMANUFACTURER.CODE   as ProductManufacturer
       from 
          DB1_PRODUCT TPRODUCT,
          DB1_AGENT   TSUPPLIER,
          DB1_AGENT   TMANUFACTURER
       where 
          TPRODUCT.ID = ProductID and 
          TSUPPLIER.ID (+) = TPRODUCT.SUPPLIER and 
          TMANUFACTURER.ID (+) = TPRODUCT.MANUFACTURER )
   loop
       HTP.print(' <product>');
       HTP.print('  <code>'        || c1.ProductCode                || '</code>');
       HTP.print('  <name>'        || DBMS_XMLGEN.convert(c1.ProductName)    || '</name>');
       HTP.print('  <article>'     || DBMS_XMLGEN.convert(c1.ProductArticle) || '</article>');
       HTP.print('  <supplier>'    || c1.ProductSupplier            || '</supplier>');
       HTP.print('  <manufacturer>'|| c1.ProductManufacturer        || '</manufacturer>');
       HTP.print('  <taxes>'       || NDSDescription(c1.ProductNDS) || '</taxes>');
       HTP.print('  <catalog>'     || PrintCatalog(c1.ProductGroup) || '</catalog>' );
       HTP.print('   <barcodes>');
                     Barcodes(ProductID);
                     if (c1.SourceProduct is not null) then
                         Barcodes(c1.SourceProduct);
                     end if;
       HTP.print('   </barcodes>');
       HTP.print(' </product>');
   end loop;
end Product;


PROCEDURE Barcodes(ProductID in raw) is
begin
   for c3 in (
       select CODE as Barcode from DB1_PRODUCT where PID = ProductID and CLASS=14745603)
   loop
       HTP.print('    <barcode>' || c3.Barcode || '</barcode>');
   end loop;
end Barcodes;


FUNCTION PrintCatalog(ClassifID in raw) return varchar2 is
   CatCodes DB1_CLASSIF.CODE%TYPE;
begin
   for c4 in 
      (select CODE from DB1_CLASSIF connect by prior PID=ID START WITH ID=ClassifID ORDER BY LEVEL DESC)
   loop
       CatCodes := CatCodes || '.' || c4.CODE;
   end loop;

   return SUBSTR(CatCodes,2,20);
end;


------------------------------------------------------------------------------------------
--                                     
------------------------------------------------------------------------------------------
PROCEDURE GetOrder(DocID in raw) is
begin
   for c1 in (
       select 
         TORDER.CODE      as OrderCode,
         TORDER.DOC_DATE  as OrderDate, 
         TORDER.F56033331 as OrderEndpoint, 
         TDEPARTMENT.CODE as OrderDeptCode,
         TPARTNER.CODE    as OrderPartnerCode,
         TPARTNER.NAME    as OrderPartnerName,
         decode(TORDER.F28835862,HEXTORAW('01B8001801B80002'),'logist',
                                 HEXTORAW('01B8001801B80001'),'pickup',
                                 HEXTORAW('01B8001801B80003'),'courier',
                                 HEXTORAW('01B8001801B80004'),'instore',
                                 TORDER.F28835862) as OrderWay,
         decode(TORDER.F14352406,HEXTORAW('0011000301B50005'),' ',
                                 HEXTORAW('0011000303CD0005'),'  ',
                                 HEXTORAW('0011000303CD0007'),'  ',
                                 HEXTORAW('0011000303CD0001'),'',
                                 HEXTORAW('0011000303CD0002'),'',
                                 HEXTORAW('0011000301C60002'),' ', 
                                 HEXTORAW('0011000303CD0003'),' ',
                                 HEXTORAW('0011000301C60006'),'',
                                 HEXTORAW('0011000301C60005'),'',
                                 HEXTORAW('0011000303CD0004'),' ', 
                                 HEXTORAW('0011000301B50002'),'', 
                                 HEXTORAW('00110003001E0006'),'',
                                 HEXTORAW('00110003001E0001'),'',
                                 HEXTORAW('0011000302870001'),'  ',
                                 HEXTORAW('0011000302870003'),'  ',
                                 TORDER.F14352406) as OrderState
              
       from 
         DB1_DOCUMENT TORDER, 
         DB1_AGENT    TDEPARTMENT,
         DB1_AGENT    TPARTNER
       where 
         TORDER.id = DocID and 
         TORDER.PARTNER   = TPARTNER.id and
         TORDER.F15925252 = TDEPARTMENT.id) -- 
   loop
      HTP.print('<order>');
      HTP.print(' <order_id>'      || c1.OrderCode        || '</order_id>');
      HTP.print(' <date>'          || to_char(c1.OrderDate,'DD.MM.YYYY') || '</date>');
      HTP.print(' <way>'           || c1.OrderWay         || '</way>');
      HTP.print(' <endpoint>'      || c1.OrderDeptCode    || '</endpoint>');
      HTP.print(' <endpoint_from>' || c1.OrderDeptCode    || '</endpoint_from>');
      HTP.print(' <endpoint_to>'   || c1.OrderEndpoint    || '</endpoint_to>');
      HTP.print(' <customer_name>' || c1.OrderPartnerName || '</customer_name>');
      HTP.print(' <customer_phone>'|| c1.OrderPartnerCode || '</customer_phone>');
      HTP.print(' <state>'         || c1.OrderState       || '</state>');
      HTP.print(' <payment_method>'|| '</payment_method>');
      HTP.print(' <body>');  

      --  
      for c2 in (
          select 
             PRODUCTS.CODE as ProductCode,
             LINES.F14286852 as LineCount,
             LINES.F15007748 as LinePrice
          from 
             DB1_LINE LINES, 
             DB1_PRODUCT PRODUCTS
          where LINES.DOCUMENT = DocID and PRODUCTS.ID = LINES.PRODUCT)
      loop
          HTP.print('  <line>');
          HTP.print('   <code>'  || c2.ProductCode || '</code>');
          HTP.print('   <count>' || c2.LineCount   || '</count>');
          HTP.print('   <price>' || to_char(c2.LinePrice,'FM99990.00') || '</price>');
          HTP.print('  </line>');
      end loop;

      HTP.print(' </body>');
      HTP.print('</order>');
   end loop;
end;


------------------------------------------------------------------------------------------
--
--                                
--
------------------------------------------------------------------------------------------
PROCEDURE GetDocument (DocID in raw) is
begin
   for c1 in (
      select 
         TDOC.CODE as DocumentCode,
         TDOC.DOC_DATE as DocumentDate, 
         DocumentTypeDescription(TDOC.TYPE) as DocumentType,
         TDEPT.CODE as DocumentDeptCode,
         TPART.CODE as DocumentPartnerCode
      from 
         DB1_DOCUMENT TDOC, 
         DB1_AGENT    TDEPT,
         DB1_AGENT    TPART
      where 
         TDOC.id = DocID and 
         TPART.id (+) = TDOC.PARTNER and
         TDOC.DEPARTMENT = TDEPT.id)
   loop
      HTP.print('<document>');
      HTP.print(' <code>'        || c1.DocumentCode        || '</code>');
      HTP.print(' <date>'        || to_char(c1.DocumentDate,'DD.MM.YYYY') || '</date>'); 
      HTP.print(' <type>'        || c1.DocumentType        || '</type>');
      HTP.print(' <department>'  || c1.DocumentDeptCode    || '</department>');
      HTP.print(' <partner>'     || c1.DocumentPartnerCode || '</partner>');
      HTP.print(' <body>'); 

      --  
      for c2 in (
          select 
             TPRODUCT.CODE      as ProductCode,
             TPRODUCT.F14745613 as ProductNDS,      --     
             TSUPPLIER.CODE     as ProductSupplier, -- 
             TLINES.F14286852   as LineCount,
             TLINES.F15007748   as RetailPrice,
             TLINES.F15007745   as PurchasingPrice, --     
             TLINES.F15007756   as UnitCost         --      
          from 
             DB1_LINE    TLINES, 
             DB1_PRODUCT TPRODUCT,
             DB1_AGENT   TSUPPLIER
          where 
             TLINES.DOCUMENT = DocID and 
             TPRODUCT.ID = TLINES.PRODUCT and 
             TSUPPLIER.ID (+) = TPRODUCT.SUPPLIER and 
             TLINES.PID IS NOT NULL)
      loop
          HTP.print('  <line>');
          HTP.print('   <code>'            ||c2.ProductCode                           || '</code>');
          HTP.print('   <count>'           ||c2.LineCount                             || '</count>');
          HTP.print('   <purchasingprice>' ||to_char(c2.PurchasingPrice,'FM99990.00') || '</purchasingprice>');
          HTP.print('   <unitcost>'        ||to_char(c2.UnitCost,'FM99990.00')        || '</unitcost>');
          HTP.print('   <retailprice>'     ||to_char(c2.RetailPrice,'FM99990.00')     || '</retailprice>');
          HTP.print('   <taxes>'           ||NDSDescription(c2.ProductNDS)            || '</taxes>');
          HTP.print('   <supplier>'        ||c2.ProductSupplier                       || '</supplier>');
          HTP.print('  </line>');
      end loop;

      HTP.print(' </body>'); 
      HTP.print('</document>');
   end loop;
end GetDocument;



------------------------------------------------------------------------------------------
--
--                                 
--
------------------------------------------------------------------------------------------
PROCEDURE GetChilds (DocID in raw) is
begin
   HTP.print('<reference>');
   for c4 in (
       select DB1_DOCUMENT.ID
       from DB1_DOCUMENT connect by prior ID=PID START WITH PID=DocID ORDER BY LEVEL ASC)
   loop
       GetDocument(c4.ID);
   end loop;
   HTP.print('</reference>');
end GetChilds;



------------------------------------------------------------------------------------------
--
--                                   
--
------------------------------------------------------------------------------------------
PROCEDURE create_move_doc(xmldoc in clob) is
   DocID           DB1_DOCUMENT.ID%TYPE;
   OrderID         DB1_DOCUMENT.ID%TYPE;
   DeptID          DB1_AGENT.ID%TYPE;
   PartnerID       DB1_AGENT.ID%TYPE;
   LineID          DB1_LINE.ID%TYPE;
   ProductID       DB1_PRODUCT.ID%TYPE;
   PartProductID   DB1_PRODUCT.ID%TYPE;

   DocumentType    DB1_DOCUMENT.TYPE%TYPE := 14286854; --  
   DocumentDate    DB1_DOCUMENT.DOC_DATE%TYPE;
   DocumentOrder   DB1_DOCUMENT.CODE%TYPE;
   DocumentCode    DB1_DOCUMENT.CODE%TYPE;
   DocumentDept    DB1_AGENT.CODE%TYPE;
   DocumentPartner DB1_AGENT.CODE%TYPE;

   Quantity        DB1_LINE.F14286852%TYPE;
   RetailPrice     DB1_LINE.F15007748%TYPE;
   UnitCost        DB1_LINE.F15007756%TYPE;
   Purchasing      DB1_LINE.F15007745%TYPE;

begin
   writelog('create_move_doc',xmldoc);

   OWA_UTIL.mime_header('text/xml');
   HTP.print('<?xml version="1.0" encoding="windows-1251"?>');
   HTP.print('<response>');

   begin
     select
        extractvalue(VALUE(xmltbl), '/document/@order'),
        extractvalue(VALUE(xmltbl), '/document/@code'), 
to_date(extractvalue(VALUE(xmltbl), '/document/@date'),'dd.mm.yyyy'),
        extractvalue(VALUE(xmltbl), '/document/@department'),
        extractvalue(VALUE(xmltbl), '/document/@partner')
     into 
       DocumentOrder,
       DocumentCode,
       DocumentDate,
       DocumentDept,
       DocumentPartner
       from table(XMLSequence(XMLType(xmldoc).extract('/document'))) xmltbl;

       if (DocumentOrder is null) then 
           HTP.print(MetaMessage('401')); goto Ext;
       end if;
       if (DocumentCode is null) then 
           HTP.print(MetaMessage('402')); goto Ext;
       end if;
       if (DocumentDate is null) then 
           HTP.print(MetaMessage('403')); goto Ext;
       end if;
       if (DocumentDept is null) then 
           HTP.print(MetaMessage('404')); goto Ext;
       end if;
       if (DocumentPartner is null) then 
           HTP.print(MetaMessage('405')); goto Ext;
       end if;
   exception when OTHERS then
           HTP.print(MetaMessage('400')); goto Ext;
   end;

   --   
   OrderID := FindDocument(DocumentOrder,1966128); -- 
   if (OrderID is null) then     
       HTP.print(MetaMessage('301')); goto Ext;
   end if;
   --    
   DocID := FindDocument(DocumentCode, DocumentType);
   if (DocID is not null) then     
       HTP.print(MetaMessage('302')); goto Ext;
   end if;

   --     (,   )

   --  
   DeptID := GetStore(DocumentDept);
   if (DeptID is null) then 
       HTP.print(MetaMessage('304')); goto Ext;
   end if;
   --  
   PartnerID := GetPartner(DocumentPartner);
   if (PartnerID is null) then 
       HTP.print(MetaMessage('305')); goto Ext;
   end if;


   --------------------------------------------------------------------
   --     
   --------------------------------------------------------------------
   begin 
      for c1 in (
          select
             extractvalue(VALUE(mytbl), '/line/code')  as ProductCode,
             extractvalue(VALUE(mytbl), '/line/count') as Quantity,
             extractvalue(VALUE(mytbl), '/line/retailprice') as RetailPrice,
             extractvalue(VALUE(mytbl), '/line/unitcost') as UnitCost,
             extractvalue(VALUE(mytbl), '/line/purchasingprice') as Purchasing
          from table(XMLSequence(XMLType(xmldoc).extract('/document/body/line'))) mytbl)  
      loop
          if (c1.ProductCode is null) then 
              HTP.print(MetaMessage('406')); goto Ext;
          end if;

          if (c1.Quantity is null) or (c1.Quantity <= 0)then 
              HTP.print(MetaMessage('407')); goto Ext;
          end if;
               
          PartProductID := GetPartProduct(c1.ProductCode);
          if (PartProductID is null) then 
              HTP.print(MetaMessage('306')); goto Ext;
          end if;

          if (GetBaseProduct(PartProductID) is null) then 
              HTP.print(MetaMessage('307')); goto Ext;
          end if;

          --            
      end loop;
   exception when OTHERS then
      HTP.print(MetaMessage('400')); goto Ext;
   end;


   begin 
      if ( DOMINO.GetUserID() is null ) then DOMINO.login(PickupService); end if;

      --     
      insert into DB1_DOCUMENT (CLASS, TYPE, CODE, 
         DOC_DATE, 
         PID,         -- 
         DEPARTMENT,  --     
         PARTNER,     --   ( )
         F15925252,   --  
         F1966094,    --  
         F28835862,   --  
         F42401912,   --  "  "
         F14352406,   -- 
         F56033331,   --    "EndpointId"
         F42401794)   
      values (14286851, DocumentType, DocumentCode,
         DocumentDate,
         OrderID,     -- 
         PartnerID,   --     
         DeptID,      --   ( )
         PartnerID,   
         DeptID,
         HEXTORAW('01B8001801B80001'), --   ""(Pickup)
         HEXTORAW('0011000302870001'), --  "  "
         HEXTORAW('0011000302870001'), --  "  "
         DeptID,
         DeptID) 
      returning ID into DocID;

      --   
      for c1 in (
          select
             extractvalue(VALUE(mytbl), '/line/code')  as ProductCode,
             extractvalue(VALUE(mytbl), '/line/count') as Quantity,
             extractvalue(VALUE(mytbl), '/line/retailprice') as RetailPrice,
             extractvalue(VALUE(mytbl), '/line/unitcost') as UnitCost,
             extractvalue(VALUE(mytbl), '/line/purchasingprice') as Purchasing
          from table(XMLSequence(XMLType(xmldoc).extract('/document/body/line'))) mytbl)  
      loop
          PartProductID := GetPartProduct(c1.ProductCode);
          ProductID     := GetBaseProduct(PartProductID);
          Quantity      := to_number(c1.Quantity,'999999.999');
          UnitCost      := to_number(c1.UnitCost,'99999999999.999999');
          Purchasing    := to_number(c1.Purchasing,'99999999999.999999');
          RetailPrice   := to_number(c1.RetailPrice,'99999999999.999999');

          --     
          insert into DB1_LINE 
                 (CLASS, TYPE, DOCUMENT, PRODUCT, 
                  LINE_DATE,       --  
                  F7,              --  
                  F12,             --  
                  F15007748,       -- RetailPrice 
                  F15007745,       -- Purchasing
                  F15007756,       -- UnitCost
                  F14286852)       -- Quantity
          values (14286849, 14286869, DocID, ProductID, 
                  DocumentDate,    --   
                  PartnerID,       --  
                  PartProductID,
                  RetailPrice,
                  Purchasing,
                  UnitCost,
                  Quantity)
          returning ID into LineID;

          --     
          insert into DB1_LINE 
                 (CLASS, TYPE, DOCUMENT, PRODUCT, PID,
                  LINE_DATE,       --  
                  F7,              --  
                  F15007748,       -- RetailPrice 
                  F15007745,       -- Purchasing
                  F15007756,       -- UnitCost
                  F14286852)       -- Quantity
          values (14286853, 14286863, DocID, PartProductID, LineID,
                  DocumentDate,    --   
                  PartnerID,       --  
                  RetailPrice,
                  Purchasing,
                  UnitCost,
                  Quantity);
      end loop;

      --     "  "
      begin 
         ChangeOrderState(OrderID, HEXTORAW('0011000302870001'));
      exception when OTHERS then
         HTP.print(MetaMessage('400')); goto Ext;
      end;

      --      
      insert into ACCEPTQUEUE (DOCUMENT) values (DocID);


   exception when OTHERS then
      HTP.print(MetaMessage('400')); 
      rollback;
      goto Ext;
   end;

   HTP.print(MetaMessage('0')); 
<<Ext>>
   HTP.print('</response>');
   return;
end create_move_doc;


------------------------------------------------------------------------------------------
--
--                                   
--
------------------------------------------------------------------------------------------
PROCEDURE create_sale_doc (xmldoc in clob) is
   DocID           DB1_DOCUMENT.ID%TYPE;
   OrderID         DB1_DOCUMENT.ID%TYPE;
   DeptID          DB1_AGENT.ID%TYPE;
   PartnerID       DB1_AGENT.ID%TYPE;
   LineID          DB1_LINE.ID%TYPE;
   ProductID       DB1_PRODUCT.ID%TYPE;
   PartProductID   DB1_PRODUCT.ID%TYPE;

   DocumentType    DB1_DOCUMENT.TYPE%TYPE := 1966130; -- 
   DocumentDate    DB1_DOCUMENT.DOC_DATE%TYPE;
   DocumentOrder   DB1_DOCUMENT.CODE%TYPE;
   DocumentCode    DB1_DOCUMENT.CODE%TYPE;
   DocumentDept    DB1_AGENT.CODE%TYPE;
   DocumentPartner DB1_AGENT.CODE%TYPE;
   DocumentState   varchar2(4);

   Quantity        DB1_LINE.F14286852%TYPE;
   RetailPrice     DB1_LINE.F15007748%TYPE;
   UnitCost        DB1_LINE.F15007756%TYPE;
   Purchasing      DB1_LINE.F15007745%TYPE;

begin
   writelog('create_sale_doc',xmldoc);

   OWA_UTIL.mime_header('text/xml');
   HTP.print('<?xml version="1.0" encoding="windows-1251"?>');
   HTP.print('<response>');

   begin
     select
        extractvalue(VALUE(xmltbl), '/document/@order'),
        extractvalue(VALUE(xmltbl), '/document/@code'), 
to_date(extractvalue(VALUE(xmltbl), '/document/@date'),'dd.mm.yyyy'),
        extractvalue(VALUE(xmltbl), '/document/@department'),
        extractvalue(VALUE(xmltbl), '/document/@partner'),
        extractvalue(VALUE(xmltbl), '/document/@status')
     into 
       DocumentOrder,
       DocumentCode,
       DocumentDate,
       DocumentDept,
       DocumentPartner,
       DocumentState
     from table(XMLSequence(XMLType(xmldoc).extract('/document'))) xmltbl;

     if (DocumentOrder is null) then 
         HTP.print(MetaMessage('401')); goto Ext;
     end if;
     if (DocumentCode is null) then 
         HTP.print(MetaMessage('402')); goto Ext;
     end if;
     if (DocumentDate is null) then 
         HTP.print(MetaMessage('403')); goto Ext;
     end if;
     if (DocumentDept is null) then 
         HTP.print(MetaMessage('404')); goto Ext;
     end if;
     if (DocumentPartner is null) then 
         HTP.print(MetaMessage('405')); goto Ext;
     end if;
   exception when OTHERS then
         HTP.print(MetaMessage('400')); goto Ext;
   end;

   --   
   OrderID := FindDocument(DocumentOrder,1966128); -- 
   if (OrderID is null) then     
       HTP.print(MetaMessage('301')); goto Ext;
   end if;
   --    
   DocID := FindDocument(DocumentCode, DocumentType);
   if (DocID is not null) then     
       HTP.print(MetaMessage('302')); goto Ext;
   end if;
   --  
   DeptID := GetStore(DocumentDept);
   if (DeptID is null) then 
       HTP.print(MetaMessage('304')); goto Ext;
   end if;
   --  
   PartnerID := GetPartner(DocumentPartner);
   if (PartnerID is null) then 
       HTP.print(MetaMessage('305')); goto Ext;
   end if;

   --------------------------------------------------------------------
   --     
   --------------------------------------------------------------------
   begin 
      for c1 in (
          select
             extractvalue(VALUE(mytbl), '/line/code')  as ProductCode,
             extractvalue(VALUE(mytbl), '/line/count') as Quantity,
             extractvalue(VALUE(mytbl), '/line/retailprice') as RetailPrice
          from table(XMLSequence(XMLType(xmldoc).extract('/document/body/line'))) mytbl)  
      loop
          if (c1.ProductCode is null) then 
              HTP.print(MetaMessage('406')); goto Ext;
          end if;

          if (c1.Quantity is null) or (c1.Quantity <= 0) then 
              HTP.print(MetaMessage('407')); goto Ext;
          end if;

          --      ,      .     
          if (GetProduct(c1.ProductCode) is null) then 
              HTP.print(MetaMessage('307')); goto Ext;
          end if;

          --       .

      end loop;
   exception when OTHERS then
      HTP.print(MetaMessage('400')); goto Ext;
   end;


   begin 
      if ( DOMINO.GetUserID() is null ) then DOMINO.login(PickupService); end if;

      --   
      insert into DB1_DOCUMENT (CLASS, TYPE, CODE, STATE,
         DOC_DATE, 
         PID,         -- 
         DEPARTMENT,  --   ( )
         PARTNER,     
         F42401912,   -- ,  
         F56033331)   --    "EndpointId"
      values (14286851, DocumentType, DocumentCode, 1,
         DocumentDate,
         OrderID,     -- 
         DeptID,      --   ( )
         PartnerID,   
         HEXTORAW('0197000201970008'),
         DeptID) 
      returning ID into DocID;

      --   
      for c1 in (
            select
               extractvalue(VALUE(mytbl), '/line/code')  as ProductCode,
               extractvalue(VALUE(mytbl), '/line/count') as Quantity,
               extractvalue(VALUE(mytbl), '/line/retailprice') as RetailPrice
            from table(XMLSequence(XMLType(xmldoc).extract('/document/body/line'))) mytbl)  
      loop
            ProductID     := GetProduct(c1.ProductCode);
            Quantity      := to_number(c1.Quantity,'999999.999');
            RetailPrice   := to_number(c1.RetailPrice,'99999999999.999999');

            --    
            insert into DB1_LINE 
                   (CLASS, TYPE, DOCUMENT, PRODUCT, 
                    LINE_DATE,       --  
                    F7,              --  
                    F15007748,       -- RetailPrice 
                    F14286852)       -- Quantity
            values (14286849, 1966229, DocID, ProductID, 
                    DocumentDate,    --   
                    DeptID,          --  
                    RetailPrice,
                    Quantity);
      end loop;


      --   
      begin 
         if (DocumentState = '2') then 
            --    ( )
            ChangeOrderState(OrderID, HEXTORAW('0011000303CD0005'));
         else
            --  
            ChangeOrderState(OrderID, HEXTORAW('0011000301B50005'));
         end if;
      exception when OTHERS then
         HTP.print(MetaMessage('400')); goto Ext;
      end;

      --      
      --   
      -- insert into ACCEPTQUEUE (DOCUMENT) values (DocID);

   exception when OTHERS then
      HTP.print(MetaMessage('400')); 
      rollback;
      goto Ext;
   end;

   HTP.print(MetaMessage('0')); 
<<Ext>>
   HTP.print('</response>');
   return;
end create_sale_doc;


------------------------------------------------------------------------------------------
--
--                                
--
------------------------------------------------------------------------------------------
PROCEDURE create_claim_doc (xmldoc in clob) is
   DocID              DB1_DOCUMENT.ID%TYPE;
   OrderID         DB1_DOCUMENT.ID%TYPE;
   DeptID          DB1_AGENT.ID%TYPE;
   PartnerID       DB1_AGENT.ID%TYPE;
   LineID          DB1_LINE.ID%TYPE;
   ProductID       DB1_PRODUCT.ID%TYPE;
   PartProductID   DB1_PRODUCT.ID%TYPE;
   Comments        DB1_DOCUMENT.F11%TYPE;
   BaseProductCode DB1_PRODUCT.CODE%TYPE;
   DotPos          Number (5,0);

   DocumentType       DB1_DOCUMENT.TYPE%TYPE :=62455909; -- 
   DocumentDate       DB1_DOCUMENT.DOC_DATE%TYPE;
   DocumentOrder      DB1_DOCUMENT.CODE%TYPE;
   DocumentCode       DB1_DOCUMENT.CODE%TYPE;
   DocumentParentCode DB1_DOCUMENT.CODE%TYPE;
   DocumentDept       DB1_AGENT.CODE%TYPE;
   DocumentPartner    DB1_AGENT.CODE%TYPE;


   Quantity           DB1_LINE.F14286852%TYPE;
   RetailPrice        DB1_LINE.F15007748%TYPE;
   UnitCost           DB1_LINE.F15007756%TYPE;
   Purchasing         DB1_LINE.F15007745%TYPE;
begin
   writelog('create_claim_doc',xmldoc);

   OWA_UTIL.mime_header('text/xml');
   HTP.print('<?xml version="1.0" encoding="windows-1251"?>');
   HTP.print('<response>');

   begin
     select
        extractvalue(VALUE(xmltbl), '/document/@order'),
        extractvalue(VALUE(xmltbl), '/document/@code'), 
        extractvalue(VALUE(xmltbl), '/document/@parent'), 
to_date(extractvalue(VALUE(xmltbl), '/document/@date'),'dd.mm.yyyy'),
        extractvalue(VALUE(xmltbl), '/document/@department'),
        extractvalue(VALUE(xmltbl), '/document/@partner'),
        extractvalue(VALUE(xmltbl), '/document/@comment')
     into 
       DocumentOrder,
       DocumentCode,
       DocumentParentCode,
       DocumentDate,
       DocumentDept,
       DocumentPartner,
       Comments
     from table(XMLSequence(XMLType(xmldoc).extract('/document'))) xmltbl;
 
     if (DocumentOrder is null) then 
         HTP.print(MetaMessage('401')); goto Ext;
     end if;
     if (DocumentCode is null) then 
         HTP.print(MetaMessage('402')); goto Ext;
     end if;
     if (DocumentDate is null) then 
         HTP.print(MetaMessage('403')); goto Ext;
     end if;
     if (DocumentDept is null) then 
         HTP.print(MetaMessage('404')); goto Ext;
     end if;
     if (DocumentPartner is null) then 
         HTP.print(MetaMessage('405')); goto Ext;
     end if;

   exception when OTHERS then
         HTP.print(MetaMessage('400')); goto Ext;
   end;


   --   
   OrderID := FindDocument(DocumentOrder,1966128); -- 
   if (OrderID is null) then     
       HTP.print(MetaMessage('301')); goto Ext;
   end if;
   --    
   DocID := FindDocument(DocumentCode, DocumentType);
   if (DocID is not null) then     
       HTP.print(MetaMessage('302')); goto Ext;
   end if;
   --  
   DeptID := GetStore(DocumentDept);
   if (DeptID is null) then 
       HTP.print(MetaMessage('304')); goto Ext;
   end if;
   --  
   PartnerID := GetStore(DocumentPartner);
   if (PartnerID is null) then 
       HTP.print(MetaMessage('305')); goto Ext;
   end if;


   --------------------------------------------------------------------
   --     
   --------------------------------------------------------------------
   begin 
      for c1 in (
          select
             extractvalue(VALUE(mytbl), '/line/code')  as ProductCode,
             extractvalue(VALUE(mytbl), '/line/count') as Quantity,
             extractvalue(VALUE(mytbl), '/line/retailprice') as RetailPrice,
             extractvalue(VALUE(mytbl), '/line/unitcost') as UnitCost,
             extractvalue(VALUE(mytbl), '/line/purchasingprice') as Purchasing
          from table(XMLSequence(XMLType(xmldoc).extract('/document/body/line'))) mytbl)  
      loop
          if (c1.ProductCode is null) then 
              HTP.print(MetaMessage('406')); goto Ext;
          end if;

          if (c1.Quantity is null) or (c1.Quantity = 0) then 
              HTP.print(MetaMessage('407')); goto Ext;
          end if;
              
          PartProductID := GetPartProduct(c1.ProductCode);

          --         
          if (PartProductID is null) then 
              DotPos := INSTR (c1.ProductCode, '.', 1, 1);

              if (DotPos <= 0) then
                  HTP.print(MetaMessage('306')); goto Ext;
              end if;

              BaseProductCode := SUBSTR (c1.ProductCode, 1, DotPos - 1);
              if (GetProduct(BaseProductCode) is null) then 
                  HTP.print(MetaMessage('307')); goto Ext;
              end if;
          else
              if (GetBaseProduct(PartProductID) is null) then 
                  HTP.print(MetaMessage('307')); goto Ext;
              end if;
          end if;

      end loop;

   exception when OTHERS then
      HTP.print(MetaMessage('400')); goto Ext;
   end;


   --   ,    
   begin 
      if ( DOMINO.GetUserID() is null ) then DOMINO.login(PickupService); end if;

      --  
      insert into DB1_DOCUMENT (CLASS, TYPE, CODE, DOC_DATE, STATE,
         PID,         -- 
         DEPARTMENT,  
         PARTNER,
         PARTN_DOC,
         F42401912,
         F14352406,   --  " " 
         F11)
      values (15728641, DocumentType, DocumentCode, DocumentDate, 1,
         OrderID,     -- 
         DeptID,      
         PartnerID,      
         DocumentParentCode,
         HEXTORAW('0011000301B50001'),
         HEXTORAW('0011000301B50001'), --  " "
         Comments) 
      returning ID into DocID;

      --   
      for c1 in (
          select
             extractvalue(VALUE(mytbl), '/line/code')  as ProductCode,
             extractvalue(VALUE(mytbl), '/line/count') as Quantity,
             extractvalue(VALUE(mytbl), '/line/retailprice') as RetailPrice,
             extractvalue(VALUE(mytbl), '/line/unitcost') as UnitCost,
             extractvalue(VALUE(mytbl), '/line/purchasingprice') as Purchasing
          from table(XMLSequence(XMLType(xmldoc).extract('/document/body/line'))) mytbl)  
      loop
          PartProductID := GetPartProduct(c1.ProductCode);

          --         
          if (PartProductID is null) then 
              DotPos := INSTR (c1.ProductCode, '.', 1, 1);
              BaseProductCode := SUBSTR (c1.ProductCode, 1, DotPos - 1);
              ProductID := GetProduct(BaseProductCode);
          else
              ProductID := GetBaseProduct(PartProductID);
          end if;

          Quantity      := to_number(c1.Quantity,'999999.999');
          UnitCost      := to_number(c1.UnitCost,'99999999999.999999');
          Purchasing    := to_number(c1.Purchasing,'99999999999.999999');
          RetailPrice   := to_number(c1.RetailPrice,'99999999999.999999');

          --     
          insert into DB1_LINE 
                 (CLASS, TYPE, DOCUMENT, PRODUCT, 
                  LINE_DATE,       --  
                  F7,              --  
                  F12,             --  
                  F15007748,       -- RetailPrice 
                  F15007745,       -- Purchasing
                  F15007756,       -- UnitCost
                  F14286852)       -- Quantity
          values (14286849, 62455910, DocID, ProductID, 
                  DocumentDate,    --   
                  DeptID,          --  
                  PartProductID,
                  RetailPrice,
                  Purchasing,
                  UnitCost,
                  Quantity)
          returning ID into LineID;

          --     
          insert into DB1_LINE 
                 (CLASS, TYPE, DOCUMENT, PRODUCT, PID,
                  LINE_DATE,       --  
                  F7,              --  
                  F15007748,       -- RetailPrice 
                  F15007745,       -- Purchasing
                  F15007756,       -- UnitCost
                  F14286852)       -- Quantity
          values (14286852, 56033414, DocID, PartProductID, LineID,
                  DocumentDate,    --   
                  DeptID,          --  
                  RetailPrice,
                  Purchasing,
                  UnitCost,
                  Quantity);
      end loop;

   exception when OTHERS then
      HTP.print(MetaMessage('400')); 
      rollback;
      goto Ext;
   end;

   HTP.print(MetaMessage('0')); 

<<Ext>>
   HTP.print('</response>');
   return;
end create_claim_doc;



PROCEDURE revoke_pickup (xmldoc in clob) is
   OrderID         DB1_DOCUMENT.ID%TYPE;
   DeptID          DB1_AGENT.ID%TYPE;
   Comments        DB1_DOCUMENT.F11%TYPE;

   DocumentOrder   DB1_DOCUMENT.CODE%TYPE;
   DocumentDept    DB1_AGENT.CODE%TYPE;
begin
   writelog('revoke_order',xmldoc);

   OWA_UTIL.mime_header('text/xml');
   HTP.print('<?xml version="1.0" encoding="windows-1251"?>');
   HTP.print('<response>');

   begin
     select
       extractvalue(VALUE(xmltbl), '/order/@order_id'),
       extractvalue(VALUE(xmltbl), '/order/@department'),
       extractvalue(VALUE(xmltbl), '/order/@comment')
     into 
       DocumentOrder,
       DocumentDept,
       Comments
     from table(XMLSequence(XMLType(xmldoc).extract('/order'))) xmltbl;
 
     if (DocumentOrder is null) then 
         HTP.print(MetaMessage('401')); goto Ext;
     end if;
     if (DocumentDept is null) then 
         HTP.print(MetaMessage('404')); goto Ext;
     end if;
   exception when OTHERS then
         HTP.print(MetaMessage('400')); goto Ext;
   end;

   --   
   OrderID := FindDocument(DocumentOrder,1966128); -- 
   if (OrderID is null) then     
       HTP.print(MetaMessage('301')); goto Ext;
   end if;

   --  
   DeptID := GetStore(DocumentDept);
   if (DeptID is null) then 
       HTP.print(MetaMessage('304')); goto Ext;
   end if;


   --     "  " 0011000302870003
   begin 
      ChangeOrderState(OrderID, HEXTORAW('0011000302870003'));
   exception when OTHERS then
      HTP.print(MetaMessage('400')); goto Ext;
   end;

   HTP.print(MetaMessage('0')); 
<<Ext>>
   HTP.print('</response>');
   return;
end revoke_pickup;



PROCEDURE get_img_orders (xmldoc in clob) is
   ReportDate      DATE; 
begin
   OWA_UTIL.mime_header('text/xml');
   HTP.print('<?xml version="1.0" encoding="windows-1251"?>');
   HTP.print('<response>');

   begin
     select
       to_date(extractvalue(VALUE(xmltbl), '/report/@date'),'dd.mm.yyyy')
       into  ReportDate
       from table(XMLSequence(XMLType(xmldoc).extract('/report'))) xmltbl;
   exception when OTHERS then
       HTP.print(MetaMessage('400')); goto Ext;
   end;

   if (ReportDate is null) then 
       HTP.print(MetaMessage('403')); goto Ext;
   end if;

   for c1 in (
      select 
        doc.CODE      as DocumentCode,
        doc.DOC_DATE  as DocumentDate,
        doc.F14352406 as DocumentState,
        decode(doc.F14352406,HEXTORAW('0011000301B50005'),' ',
                             HEXTORAW('0011000303CD0005'),'  ',
                             HEXTORAW('0011000303CD0007'),'  ',
                             HEXTORAW('0011000303CD0001'),'',
                             HEXTORAW('0011000303CD0002'),'',
                             HEXTORAW('0011000301C60002'),' ', 
                             HEXTORAW('0011000303CD0003'),' ',
                             HEXTORAW('0011000301C60006'),'',
                             HEXTORAW('0011000301C60005'),'',
                             HEXTORAW('0011000303CD0004'),' ', 
                             HEXTORAW('0011000301B50002'),'', 
                             HEXTORAW('00110003001E0006'),'',
                             HEXTORAW('00110003001E0001'),'',
                             HEXTORAW('0011000302870001'),'  ',
                             HEXTORAW('0011000302870003'),'  ',
                             doc.F14352406) as DocumentStateDescription

      from DB1_DOCUMENT doc
      where doc.TYPE=1966128 and doc.DOC_DATE>=trunc(ReportDate) and doc.DOC_DATE<=trunc(ReportDate+1)
      )
   loop
      HTP.print('  <document>');
      HTP.print('   <code>'  || c1.DocumentCode  || '</code>');
      HTP.print('   <date>'  || to_char(c1.DocumentDate,'DD.MM.YYYY') || '</date>');
      HTP.print('   <state>' || c1.DocumentStateDescription || '</state>');
      HTP.print('  </document>');
   end loop;

<<Ext>>
   HTP.print('</response>');
   return;

exception when OTHERS then
   HTP.print(MetaMessage('400')); 
   HTP.print('</response>');
   return;
end get_img_orders;



------------------------------------------------------------------------------------------
PROCEDURE get_img_order(xmldoc in clob) is
   DocumentOrder   DB1_DOCUMENT.CODE%TYPE;
   OrderID         DB1_DOCUMENT.ID%TYPE;
   fullmode        number;
begin
   writelog('get_img_order',xmldoc);
   OWA_UTIL.mime_header('text/xml');
   HTP.print('<?xml version="1.0" encoding="windows-1251"?>');
   HTP.print('<response>');

   --   
   begin
     select
       extractvalue(VALUE(xmltbl), '/document/@code'), 
       extractvalue(VALUE(xmltbl), '/document/@fullmode') 
     into 
       DocumentOrder,
       fullmode
     from table(XMLSequence(XMLType(xmldoc).extract('/document'))) xmltbl;

   exception when OTHERS then
       HTP.print(MetaMessage('400')); goto Ext;
   end;

   --   
   OrderID := FindDocument(DocumentOrder,1966128); -- 
   if (OrderID is null) then     
       HTP.print(MetaMessage('301')); goto Ext;
   end if;

   Products(OrderID);       --    
   GetOrder(OrderID);       --   

   if (fullmode=1) then
       GetChilds(OrderID);  --   
   end if;

   HTP.print(MetaMessage('0')); 

<<Ext>>
   HTP.print('</response>');
   return;

exception when OTHERS then
   HTP.print(MetaMessage('400')); 
   HTP.print('</response>');
   return;
END get_img_order;



------------------------------------------------------------------------------------------------
--
--                                            
--
------------------------------------------------------------------------------------------------

FUNCTION GetRegion(RegionCode in varchar2) return raw is
   RegionID DB1_AGENT.ID%TYPE;
begin
   begin
     select id into RegionID
     from DB1_AGENT
     where 
       CLASS = 2 and 
       TYPE in (14745602,15073286,40566785,40566786) and 
       F15073296 = RegionCode;
   exception
       when OTHERS then RegionID := NULL;
   end;
   return RegionID;
end;


FUNCTION GetStore(StoreCode in varchar2) return raw is
   StoreID DB1_AGENT.ID%TYPE;
begin
   begin
     select id into StoreID
     from DB1_AGENT
     where 
       CLASS = 2 and             -- 
       TYPE = 14745607 and       --  /
       CODE = StoreCode;         -- F15073296=StoreCode;
   exception
       when OTHERS then  StoreID := NULL;
   end;
   return StoreID;
end;


FUNCTION GetPartner(PartnerCode in varchar2) return raw is
   PartnerID DB1_AGENT.ID%TYPE;
begin
   begin
     select id into PartnerID
     from DB1_AGENT
     where 
       CLASS in (14745601,14745602) and   --  
--       TYPE = 14745610 and    --  
       CODE = PartnerCode;    -- F15073296 = PartnerCode;
   exception
       when OTHERS then PartnerID := NULL;
   end;
   return PartnerID;
end;


FUNCTION GetClient(ClientCode in varchar2) return raw is
   ClientID DB1_AGENT.ID%TYPE;
begin
   begin
     select id into ClientID
     from DB1_AGENT
     where 
       CLASS = 14745602 and   -- 
       TYPE = 63766529 and    --  -
       CODE = ClientCode;
   exception                        
       when OTHERS then ClientID := NULL;
   end;
   return ClientID;
end;


--     
FUNCTION GetPartProduct(ProductCode in varchar2) return raw is
   ProductID DB1_PRODUCT.ID%TYPE;
begin
   begin
     select id into ProductID
     from DB1_PRODUCT
     where CLASS = 14745602 and CODE = ProductCode;
   exception                        
       when OTHERS then ProductID := NULL;
   end;
   return ProductID;
end;


--     
FUNCTION GetProduct(ProductCode in varchar2) return raw is
   ProductID DB1_PRODUCT.ID%TYPE;
begin
   begin
     select id into ProductID
     from DB1_PRODUCT
     where CLASS in (14745601,3342339) and CODE = ProductCode;
   exception                        
       when OTHERS then ProductID := NULL;
   end;
   return ProductID;
end;


--     
FUNCTION GetBaseProduct(PartProductID in raw) return raw is
   ProductID DB1_PRODUCT.ID%TYPE;
begin
   begin
      select pid into ProductID
      from DB1_PRODUCT
      where ID = PartProductID;
   exception                        
      when OTHERS then ProductID := NULL;
   end;
   return ProductID;
end;


FUNCTION DocumentTypeDescription(DocType in number) return varchar2 is
begin
   case DocType
      when 1966130  then return '';
      when 14286854 then return ' ';
      when 14286903 then return ' ';
      when 63766534 then return '';
      when 62455909 then return '';
      else return NULL;
   end case;
end;

FUNCTION NDSDescription(NDS in raw) return varchar2 is
begin 
   case NDS
      when HEXTORAW('07D2000501B70003') then return '0';
      when HEXTORAW('07D2000501B70002') then return '10';
      when HEXTORAW('07D2000502870001') then return '18';
      else return RAWTOHEX(NDS);
   end case;
end;


FUNCTION FindDocument(DocCode in varchar2, DocType in number) return raw is
   DocID raw(8);
begin
   begin
      select doc.id into DocID
      from DB1_DOCUMENT doc
      where doc.TYPE=DocType and doc.CODE=DocCode;
   exception
      when OTHERS then DocID := NULL;
   end;
   return DocID;
end FindDocument;


END pickup;

--    (   ,      )

/
SHOW ERRORS

exit;
