-------------------------------------------------------------------------------
--
--         zakaz_package.sql
--
-------------------------------------------------------------------------------
--    () 
--   HEXTORAW('00110003001E0001') --  ()
--   HEXTORAW('0011000301B50002') --      (  )
--   HEXTORAW('00110003001E0006') -- 
--   HEXTORAW('0011000303CD0001') -- 
--   HEXTORAW('0011000303CD0003') --  
--   HEXTORAW('0011000303CD0004') --  
--   HEXTORAW('0011000303CD0002') --  ()
--   HEXTORAW('0011000301C60006') --                     
--   HEXTORAW('0011000302870001') --                 
--   HEXTORAW('0011000301B50005') --    ()
--   HEXTORAW('0011000303CD0005') --    ( )
--   HEXTORAW('0011000303CD0007') --             
--   HEXTORAW('0011000301C60005') --                  
--   HEXTORAW('0011000302870003') --   
-------------------------------------------------------------------------------

create or replace package zakaz as 

  valVersion       varchar2(4)   := '5.02';

  valClientID      RAW(8)        := NULL;
  valZakazID       RAW(8)        := NULL;
  valLineID        RAW(8)        := NULL;
  valErrorMessage  varchar2(800) := NULL;

  valNoID          RAW(8)        := HEXTORAW('07D2000307D20001'); --  ""
  valYesID         RAW(8)        := HEXTORAW('07D2000307D20002'); --  ""


  FUNCTION GetRegion(RegionCode in varchar2) return raw;
  FUNCTION GetStore(StoreCode in varchar2) return raw;
  FUNCTION GetClient(ClientCode in varchar2) return raw;
  FUNCTION GetLogistic(ClientCode in varchar2) return raw;
  FUNCTION GetOrder(OrderCode in varchar2, OrderState out raw, OrderWay out raw) return raw;
  FUNCTION WayTypeDescription(Delivery in raw) return varchar2;
  FUNCTION DecodeOrderState(OrderState in raw) return varchar2;

  PROCEDURE writelog(text in varchar2);
  PROCEDURE PrintErrorMessage(text in varchar2);

  FUNCTION GetErrorMessage(error_type in varchar2) return varchar2;
  FUNCTION GetOkMessage(typeMessage in varchar2, order_id in varchar2) return varchar2;
  FUNCTION GetVersion return varchar2;

  pragma RESTRICT_REFERENCES (GetRegion, WNDS, WNPS, RNPS); 
  pragma RESTRICT_REFERENCES (GetStore, WNDS, WNPS, RNPS); 
  pragma RESTRICT_REFERENCES (GetClient, WNDS, WNPS, RNPS); 
  pragma RESTRICT_REFERENCES (GetLogistic, WNDS, WNPS, RNPS); 
  pragma RESTRICT_REFERENCES (GetOrder, WNDS, WNPS, RNPS); 
  pragma RESTRICT_REFERENCES (GetOkMessage, WNDS, WNPS, RNPS); 
  pragma RESTRICT_REFERENCES (DecodeOrderState, WNDS, RNDS, WNPS, RNPS); 

  PROCEDURE CreateClient(
            FIO        in varchar2, 
            Phone      in varchar2,
            EMail      in varchar2,
            Address    in varchar2, 
            Province   in varchar2,
            City       in varchar2,
            Street     in varchar2,
            StreetAbbr in varchar2, 
            House      in varchar2,
            Housekorp  in varchar2,
            Housestr   in varchar2,
            Flat       in varchar2);

  PROCEDURE UpdateClient(
            ClienID    in raw,
            FIO        in varchar2,
            EMail      in varchar2,
            Address    in varchar2, 
            Province   in varchar2,
            City       in varchar2,
            Street     in varchar2,
            StreetAbbr in varchar2, 
            House      in varchar2,
            Housekorp  in varchar2,
            Housestr   in varchar2,
            Flat       in varchar2);

  PROCEDURE CreateOrder(
            OrderCode        in varchar2,
            StoreID          in raw,
            ClientID         in raw,
            DeliveryWay      in varchar2,
            DeliveryDate     in Date,
            DeliveryPeriod   in varchar2,
            ShippingDate     in Date,
            PaymentType      in varchar2,
            Comments         in varchar2,
            Endpoint_to      in varchar2,
            EndpointID       in raw,
            Address          in varchar2,
            Region           in varchar2);

  PROCEDURE UpdateOrder(
            OrderID          in raw, 
            StoreID          in raw,
            ClientID         in raw,
            DeliveryWay      in varchar2,
            DeliveryDate     in Date,
            DeliveryPeriod   in varchar2,
            ShippingDate     in Date,
            PaymentType      in varchar2,
            Comments         in varchar2,
            Endpoint_to      in varchar2,
            EndpointID       in raw,
            LogisticID       in raw,
            Address          in varchar2,
            Region           in varchar2,
            StorageGroup     in varchar2);

  PROCEDURE RevokeOrder(
            OrderID          in raw,
            Reason           in varchar2);

  PROCEDURE CreateLine(
            OrderID          in raw, 
            OrderDept        in raw, 
            OrderDate        in date, 
            StoreID          in raw, 
            ProductCode      in varchar2,
            price            in number,
            qty              in number,
            linenumber       in number,
            prod_volume      in number,
            prod_weight      in number,
            RRFlag           in number);

  procedure SetZakazState8 (
            ZakazID          in raw, 
            StoreID          in raw, 
            xmldoc           in clob,
            RRFlag           in number);
  

  --  API --
  PROCEDURE reserve(data in clob);           --      .
  PROCEDURE approve(data in clob);           --      .
  PROCEDURE change(data in clob);            --    .
  PROCEDURE revoke_(data in clob);           --   
  PROCEDURE getorderstate(data in clob);     --    ,
  PROCEDURE presence(data in clob);          --    ,
  PROCEDURE price(data in clob);             --     .
  PROCEDURE about; 

END zakaz;

--    (   ,      )

/
SHOW ERRORS

-------------------------------------------------------------------------------------------


create or replace package body zakaz as

FUNCTION GetVersion return varchar2 is
begin
   return valVersion;
end;

PROCEDURE about IS
begin
   OWA_UTIL.mime_header('text/xml');  
   HTP.print('<?xml version="1.0"?>');
   HTP.print('<package name="DOMINO8.ZAKAZ" version="' || valVersion ||'" remoteip="'||OWA_UTIL.get_cgi_env('REMOTE_ADDR')||'"/>');
end;

PROCEDURE CreateClient(
   FIO        in varchar2, 
   Phone      in varchar2,
   EMail      in varchar2,
   Address    in varchar2, 
   Province   in varchar2,
   City       in varchar2,
   Street     in varchar2,
   StreetAbbr in varchar2, 
   House      in varchar2,
   Housekorp  in varchar2,
   Housestr   in varchar2,
   Flat       in varchar2) is
begin
   if ( DOMINO.GetUserID() is null ) then DOMINO.login(HEXTORAW('8400000000000001')); end if;

   insert into DB1_AGENT ( CLASS, TYPE, 
       CODE, 
       NAME, 
       F14745618, 
       F14745623, 
       F14745635, 
       F14745629,
       F14745630,
       F14745631,
       F15859714,
       F14745632,
       F14745633,
       F15859713,
       F14745634)
   values ( 14745602, 63766529,
       NVL(Substr(Phone,1,20),'+79xxxxxxxxx'), 
       Substr(FIO,1,80),
       NVL(Substr(Phone,1,40),'+79xxxxxxxxx'),
       Substr(EMail,1,40), 
       Substr(Address,1,120),
       Substr(Province,1,40),
       Substr(City,1,40),
       Substr(Street,1,40),
       Substr(StreetAbbr,1,40),
       Substr(House,1,40),
       Substr(Housekorp,1,40),
       Substr(Housestr,1,40),
       Substr(Flat,1,40)
   ) returning ID into valClientID;
   commit work;
end;


PROCEDURE UpdateClient(
   ClienID    in raw,
   FIO        in varchar2,
   EMail      in varchar2,
   Address    in varchar2, 
   Province   in varchar2,
   City       in varchar2,
   Street     in varchar2,
   StreetAbbr in varchar2, 
   House      in varchar2,
   Housekorp  in varchar2,
   Housestr   in varchar2,
   Flat       in varchar2) is
begin
   if ( DOMINO.GetUserID() is null ) then  DOMINO.login(HEXTORAW('8400000000000001'));  end if;

   --       
   update DB1_AGENT
     set NAME 	   = SubStr(FIO,1,80),
         F14745623 = SubStr(EMail,1,40),
         F14745635 = SubStr(Address,1,120),
         F14745629 = Substr(Province,1,40),
         F14745630 = SubStr(City,1,40),
         F14745631 = SubStr(Street,1,40),
         F15859714 = SubStr(StreetAbbr,1,40),
         F14745632 = SubStr(House,1,40),
         F14745633 = SubStr(Housekorp,1,40),
         F15859713 = SubStr(Housestr,1,40),
         F14745634 = SubStr(Flat,1,40),
         F14745609 = null  --     .           .
   where id = ClienID;
   commit work;
end;


PROCEDURE CreateOrder(
   OrderCode      in varchar2,
   StoreID        in raw,
   ClientID       in raw,
   DeliveryWay    in varchar2,
   DeliveryDate   in Date,
   DeliveryPeriod in varchar2,
   ShippingDate   in Date,
   PaymentType    in varchar2,
   Comments       in varchar2,
   Endpoint_to    in varchar2,
   EndpointId     in raw,
   Address        in varchar2,
   Region         in varchar2)
is
   DepatmentID    DB1_AGENT.PID%TYPE;
   CompanyID      DB1_AGENT.PID%TYPE;
begin
   if ( DOMINO.GetUserID() is null ) then DOMINO.login(HEXTORAW('8400000000000001')); end if;

   Select PID into DepatmentID from DB1_AGENT where ID = StoreID;
   Select F28835910 into CompanyID from DB1_AGENT where ID = DepatmentID;

   insert into DB1_DOCUMENT (CLASS, TYPE, CODE, DOC_DATE, 
      DEPARTMENT,  --   StoreID
      F42401919,   --    DepatmentID
      F6684704,    --  /   ""
      PARTNER,     --  "ClientID"
      F15925252,   --  "StoreID"
      F1966094,    --  "ClientID"
      F56033331,   --    "Endpoint_to"
      F42401794,   --    "EndpointID"
      F14286886,   -- , 
      F14352404,   --  
      F15073281,   --   "paymentType"
      F28835862,   --   "deliveryWay"
      F14286863,   --     (  "")
      F14352406,   --   (   "")
      F14745611,   --  
      F14942214,   --  
      PARTN_DOC,   --   "deliveryPeriod"
      F14942215,   --   "shippingDate"
      F56033334,   -- 
      F56033335)   -- 
   values (14352386, 1966128, OrderCode, sysdate,
      DepatmentID,
      CompanyID,
      valYesID,
      ClientID,
      StoreID,
      ClientID,
      Endpoint_to,
      EndpointId,
      HEXTORAW('07D2000901AD0001'), 
      HEXTORAW('00020032013D0001'),
      decode(PaymentType,
             'cash',         HEXTORAW('0095000101BC0001'),
             'cashless',     HEXTORAW('0095000101BC0002'),
             'card',         HEXTORAW('0095000101BC0003'),
             'card_terminal',HEXTORAW('0095000103DE0001'),
                             HEXTORAW('0095000101BC0001')),
      decode(DeliveryWay,
             'logist' ,HEXTORAW('01B8001801B80002'),
             'pickup' ,HEXTORAW('01B8001801B80001'),
             'courier',HEXTORAW('01B8001801B80003'),
             'instore',HEXTORAW('01B8001801B80004'),
                       HEXTORAW('01B8001801B80002')),
      valYesID,
      HEXTORAW('00110003001E0001'), --   ""
      Comments,
      DeliveryDate,
      DeliveryPeriod,
      ShippingDate, 
      Address,
      Region) returning ID into valZakazID;

   insert into ZAKAZ_STATE (ZAKAZ, STATE, CHANGE_DATE, USERID, SENDED_IM) 
   values (valZakazID, HEXTORAW('00110003001E0001'), SYSDATE, DOMINO.GetUserID(),1);
end;


PROCEDURE UpdateOrder(
   OrderID          in raw, 
   StoreID          in raw,
   ClientID         in raw,
   DeliveryWay      in varchar2,
   DeliveryDate     in Date,
   DeliveryPeriod   in varchar2,
   ShippingDate     in Date,
   PaymentType      in varchar2,
   Comments         in varchar2,
   Endpoint_to      in varchar2,
   EndpointId       in raw,
   LogisticID       in raw,
   Address          in varchar2,
   Region           in varchar2,
   StorageGroup     in varchar2)
is
   DepatmentID    DB1_AGENT.PID%TYPE;
   CompanyID      DB1_AGENT.PID%TYPE;
begin
   if ( DOMINO.GetUserID() is null ) then DOMINO.login(HEXTORAW('8400000000000001')); end if;

   Select PID into DepatmentID from DB1_AGENT where ID = StoreID;
   Select F28835910 into CompanyID from DB1_AGENT where ID = DepatmentID;

   --     F6684704 -     
   --       (F14286886  F14352404)

   update DB1_DOCUMENT 
      set DEPARTMENT = DepatmentID,
          F42401919  = CompanyID,
          PARTNER    = ClientID,
          F15925252  = StoreID,
          F1966094   = ClientID,
          F15073281  = decode(PaymentType,
                             'cash'         ,HEXTORAW('0095000101BC0001'),
                             'cashless'     ,HEXTORAW('0095000101BC0002'),
                             'card'         ,HEXTORAW('0095000101BC0003'),
                             'card_terminal',HEXTORAW('0095000103DE0001'),
                                             HEXTORAW('0095000101BC0001')),
          F28835862  = decode(DeliveryWay,
                             'logist' ,HEXTORAW('01B8001801B80002'),
                             'pickup' ,HEXTORAW('01B8001801B80001'),
                             'courier',HEXTORAW('01B8001801B80003'),
                             'instore',HEXTORAW('01B8001801B80004'),
                                       HEXTORAW('01B8001801B80002')),
          F14942214 = DeliveryDate, 
          PARTN_DOC = DeliveryPeriod,       
          F14942215 = ShippingDate,         
          F14745611 = Comments,
          F56033331 = Endpoint_to,
          F42401794 = EndpointId,
          F33620503 = LogisticID,
          F15925250 = SubStr(Address,1,120),
          F56033334 = Address, 
          F56033335 = Region,
          F14417928 = StorageGroup
   where id = OrderID;
end;


PROCEDURE CreateLine(
   OrderID     in raw, 
   OrderDept   in raw, 
   OrderDate   in date, 
   StoreID     in raw, 
   ProductCode in varchar2, 
   price       in number, 
   qty         in number, 
   linenumber  in number, 
   prod_volume in number, 
   prod_weight in number,
   RRFlag      in number)
is
   ProductID   DB1_PRODUCT.ID%TYPE;
   quantity_r  number(19,3);
begin
   if ( DOMINO.GetUserID() is null ) then DOMINO.login(HEXTORAW('8400000000000001')); end if;

   begin
      select id into ProductID
      from DB1_PRODUCT
      where CLASS=14745601 and CODE=ProductCode;
   exception
      when OTHERS then 
           ProductID := NULL;
           valErrorMessage := 'Product not found. Code = ' || ProductCode;
           return;
   end;

   if (RRFlag = 1) then
       --  
       quantity_r := 0;
       Select NVL(Sum(t1.qty),0) into quantity_r
         from (
           Select qty from DB1_TRANSFER_A2_VIEW where (DEPT = StoreID) and (PRODUCT = ProductID)
           union all
           Select Sum(-amount) as qty from DB1_VRSTOCK where (DEPT = StoreID) and (PRODUCT = ProductID)
       ) t1;

       if (quantity_r > 0) then 
           if (quantity_r >= qty) then
               quantity_r := qty;
           end if;
       else
           quantity_r := 0;
       end if;
   else
       quantity_r := qty;
   end if;

   --   
   insert into DB1_LINE (CLASS, TYPE, DOCUMENT, LINE_DATE,
                        F7,
                        PRODUCT,
                        LINE_NO,
                        F15007747, F15007754, F15007748, -- Price
                        F14286852, F14286860,            -- QTY
                        F42401809,                       -- Volume
                        F2818051,                        -- Weight
                        F15138819)                       -- Discount
                values (14286849, 1966227, OrderID, OrderDate, 
                        OrderDept, 
                        ProductID,
                        linenumber,
                        price, price, price,
                        quantity_r, qty, 
                        prod_volume,
                        round(prod_weight,4),
                        0) returning ID into valLineID;

   if (valLineId is NULL)  then
       valErrorMessage := 'Error create line. Code = ' || ProductCode;
       return;
   end if;


   --   
   if (quantity_r > 0) and (RRFlag = 1) then
       insert into DB1_RESERV (DEPT,PRODUCT,REZ_DATE,DOC,LINE,A3_DOC,A4,AMOUNT)
       values (StoreID, ProductID, OrderDate, OrderID, valLineID, OrderID, valLineID, quantity_r);

       insert into DB1_RESERVE (DEPT,PRODUCT,FACT_DATE,DOCUMENT,LINE,A_DOCUMENT,A_LINE,QTY,SIGN,AGGR)
       values (StoreID, ProductID, OrderDate, OrderID, valLineID, OrderID, valLineID, quantity_r,'+','1');
   end if;

end;



PROCEDURE RevokeOrder(
  OrderID in raw, 
  Reason in varchar2) is
begin

  if ( DOMINO.GetUserID() is null ) then DOMINO.login(HEXTORAW('8400000000000001')); end if;

  update DB1_DOCUMENT 
     set F14352406 = HEXTORAW('0011000301C60005'), 
         F14745611 = Reason,
         State     = 1
     where id = OrderID;

  insert into DB1_RESERV (DEPT,PRODUCT,REZ_DATE,DOC,LINE,A3_DOC,A4,AMOUNT) (
     Select rs.DEPT, rs.PRODUCT,sysdate,rs.DOC,rs.A4,rs.DOC,rs.A4,-rs.AMOUNT
     from DB1_VRSTOCK rs where rs.DOC = OrderID
  );

  insert into DB1_RESERVE (DEPT,PRODUCT,FACT_DATE,DOCUMENT,LINE,A_DOCUMENT,A_LINE,QTY,SIGN,AGGR) (
     Select rs.DEPT, rs.PRODUCT,sysdate,rs.DOC,rs.A4,rs.DOC,rs.A4,rs.AMOUNT,'-','1'
     from DB1_VRSTOCK rs where rs.DOC = OrderID
  );

  insert into ZAKAZ_STATE (ZAKAZ, STATE, CHANGE_DATE, USERID, SENDED_IM) 
  values (OrderID, HEXTORAW('0011000301C60005'), SYSDATE, DOMINO.GetUserID(),1);
  commit work;
end;




PROCEDURE reserve(data in clob) is
   pragma AUTONOMOUS_TRANSACTION;
   RequiredReservationFlag number(5,0) := 1; 

   xmldoc         clob;
   order_id       varchar2(20);


   DeliveryWay    varchar2(20);
   endpoint       varchar2(20);
   endpoint_from  varchar2(20);
   endpoint_to    varchar2(20);

   fio            varchar2(80);
   phone          varchar2(40);
   email          varchar2(40);
   address        varchar2(120);
   region	  varchar2(40);
   city		  varchar2(120);
   street	  varchar2(40);
   streetabbr	  varchar2(40);
   house	  varchar2(40);
   housekorp      varchar2(40);
   housestr       varchar2(40);
   flat           varchar2(40);
   Comments       varchar2(2000);
   deliverydate   varchar2(20);
   deliveryPeriod varchar2(20);
   shippingdate   varchar2(20);
   paymentType    varchar2(20);

   line_num       number(5,0);
   product_id     varchar2(20);
   line_count     number(15,0);
   price	  number(15,2);
   prod_volume    number(15,6);
   prod_weight    number(15,6);


   ZakazID        raw(8);
   ClientID       raw(8);
   StoreID        raw(8);
   OrderDept      raw(8); 
   OrderDate      date;
   OrderState     raw(8);
   OrderWay       raw(8);

   EndpointID     raw(8);
   DeliveryWayID  raw(8);

   productnotfound EXCEPTION;
   exProductID     EXCEPTION;
   exLineNumber    EXCEPTION;
   exLineCount     EXCEPTION;
   exPrice         EXCEPTION;
   exDeliveryWay   EXCEPTION;
   exDelStore	   EXCEPTION;
   exDelDate	   EXCEPTION;

begin
   xmldoc := convert(data,'CL8MSWIN1251','UTF8');
   writelog('<reserve>' || Substr(to_char(xmldoc),1,1980));

   OWA_UTIL.mime_header('text/xml');
   HTP.print('<?xml version="1.0" encoding="utf-8"?>');
   HTP.print('<response>');

   --   
   begin
      select extractvalue(VALUE(mytbl), '/order/@order_id') into order_id
      from table(XMLSequence(XMLType(xmldoc).extract('/order'))) mytbl;
   exception when OTHERS then
      valErrorMessage := 'no order id';
      HTP.print(GetErrorMessage('param_error'));
      HTP.print('</response>');
      return;
   end;

   --   
   ZakazID := GetOrder(order_id,OrderState,OrderWay);
   if (ZakazID is not null) then   --      
       valErrorMessage := 'duplicate order_id =' || order_id;
       HTP.print(GetErrorMessage('order_dublicate'));
       HTP.print('</response>');
       return;
   end if;

   --   
   begin
      select
         extractvalue(VALUE(mytbl), '/delivery/@way'),
         extractvalue(VALUE(mytbl), '/delivery/@endpoint'),
         extractvalue(VALUE(mytbl), '/delivery/@endpoint_from'),
         extractvalue(VALUE(mytbl), '/delivery/@endpoint_to'),
         extractvalue(VALUE(mytbl), '/delivery/@fio'),
         extractvalue(VALUE(mytbl), '/delivery/@address'),
         extractvalue(VALUE(mytbl), '/delivery/@region'),
         extractvalue(VALUE(mytbl), '/delivery/@city'),
         extractvalue(VALUE(mytbl), '/delivery/@street'),
         extractvalue(VALUE(mytbl), '/delivery/@streetAbbr'),
         extractvalue(VALUE(mytbl), '/delivery/@house'),
         extractvalue(VALUE(mytbl), '/delivery/@houseKorpus'),
         extractvalue(VALUE(mytbl), '/delivery/@str'),
         extractvalue(VALUE(mytbl), '/delivery/@flat'),
         extractvalue(VALUE(mytbl), '/delivery/@phone'),
         extractvalue(VALUE(mytbl), '/delivery/@remark'),
         extractvalue(VALUE(mytbl), '/delivery/@email'),
         extractvalue(VALUE(mytbl), '/delivery/@deliverydate'),
         extractvalue(VALUE(mytbl), '/delivery/@delivery_period'),
         extractvalue(VALUE(mytbl), '/delivery/@shippingdate'),
         extractvalue(VALUE(mytbl), '/delivery/@payment_method')
     into 
         DeliveryWay, 
         endpoint, 
         endpoint_from, 
         endpoint_to, 
         fio, 
         address, 
         region, 
         city,
         street,
         streetabbr,
         house, 
         housekorp,
         housestr,
         flat,
         phone,
         Comments,
         email,
         deliverydate,
         deliveryPeriod,
         shippingdate,
         paymentType
     from table(XMLSequence(XMLType(xmldoc).extract('/order/delivery'))) mytbl;

     if (DeliveryWay is null)  then raise exDeliveryWay; end if;
     if (endpoint is null)     then raise exDelStore; end if;
     if (deliverydate is null) then raise exDelDate;  end if;

     exception 
        when exDeliveryWay then
             valErrorMessage := 'no delivery way';
             HTP.print(GetErrorMessage('param_error'));
             HTP.print('</response>');
             return;
        when exDelStore then
             valErrorMessage := 'no store id';
             HTP.print(GetErrorMessage('param_error'));
             HTP.print('</response>');
             return;
        when exDelDate then
             valErrorMessage := 'no delivery date';
             HTP.print(GetErrorMessage('param_error'));
             HTP.print('</response>');
             return;
        when OTHERS then
             valErrorMessage := 'no delivery parameters';
             HTP.print(GetErrorMessage('param_error'));
             HTP.print('</response>');
             return;
   end;

   --    
   StoreID := GetStore(endpoint);
   if (StoreID is null) then   --     /
       valErrorMessage := 'bad store id = ' || endpoint;
       HTP.print(GetErrorMessage('param_error'));
       HTP.print('</response>');
       return;
   end if;


   case DeliveryWay
        when 'pickup'  then
              EndpointID := GetStore(endpoint_to);
              if (EndpointID is null) then             --       
                  valErrorMessage := 'bad salespoint id = ' || endpoint_to;
                  HTP.print(GetErrorMessage('param_error'));
                  HTP.print('</response>');
                  return;
              end if;

              if (paymentType is null) then 
                  valErrorMessage := 'payment_method not defined';
                  HTP.print(GetErrorMessage('param_error'));
                  HTP.print('</response>');
                  return;
              end if;

              if (paymentType <> 'cash') then 
                  valErrorMessage := 'payment_method not allowed';
                  HTP.print(GetErrorMessage('param_error'));
                  HTP.print('</response>');
                  return;
              end if;

        when 'instore' then
              RequiredReservationFlag := 0;            --   
              EndpointID := GetStore(endpoint_to);
              if (EndpointID is null) then             --       
                  valErrorMessage := 'bad salespoint id = ' || endpoint_to;
                  HTP.print(GetErrorMessage('param_error'));
                  HTP.print('</response>');
                  return;
              end if;
	when 'logist'  then null; --      .
	when 'courier' then null;
   	else  --      
              valErrorMessage := 'bad delivery way type = ' || DeliveryWay;
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
   end case;


   if (phone is not null) then
       ClientID := GetClient(phone);
   end if;

   if (ClientID is null) then

       CreateClient(fio,phone,email,address,region, city, street, streetabbr, house, housekorp, housestr, flat);

       ClientID := valClientID;

       if (ClientID is null) then   -- send error message
           valErrorMessage := 'error create client';
           HTP.print(GetErrorMessage('server_error'));
           HTP.print('</response>');
           return;
       end if;
   else
       UpdateClient(ClientID,fio,email,address,region, city, street, streetabbr, house, housekorp, housestr, flat);
   end if;


   --  
   begin
      CreateOrder(
            order_id,
            StoreID, 
            ClientID, 
            DeliveryWay,
            to_date(deliverydate,'dd/mm/yyyy'),  
            deliveryPeriod,
            to_date(shippingDate,'dd/mm/yyyy'),
            paymentType,
            Comments,
            endpoint_to,
            EndpointID,
            address,
            region);

      --   
      ZakazID := GetOrder(order_id,OrderState,OrderWay);
      if (ZakazID is null) then --    
          rollback;
          valErrorMessage := 'error create document';
          HTP.print(GetErrorMessage('server_error'));
          HTP.print('</response>');
          return;
      end if;  

      --      (    )
      select DEPARTMENT, DOC_DATE into OrderDept, OrderDate from DB1_DOCUMENT where ID = ZakazID;

      for C1 in (
	select
          extractvalue(VALUE(mytbl), '/product/@line')       as line_num,
          extractvalue(VALUE(mytbl), '/product/@id')         as product_id,
          extractvalue(VALUE(mytbl), '/product/@count')      as line_count,
          extractvalue(VALUE(mytbl), '/product/@price')      as price,
          extractvalue(VALUE(mytbl), '/product/@weight')     as weight,
          extractvalue(VALUE(mytbl), '/product/@volume')     as volume
        from table(XMLSequence(XMLType(xmldoc).extract('/order/products/product'))) mytbl)  loop

          line_num    := to_number(C1.line_num);
          product_id  := C1.product_id;
          line_count  := to_number(C1.line_count);
          price       := to_number(C1.price,'999999999999999.99');
          prod_weight := to_number(C1.weight,'999999999999999.999999');
          prod_volume := to_number(C1.volume,'999999999999999.999999');

          if (line_num is null) then raise exLineNumber; end if;
          if (product_id is null) then raise exProductID; end if;
          if (line_count is null) then raise exLineCount; end if;
          if (price is null) then raise exPrice; end if;

          valErrorMessage := '';
          
          CreateLine(
                ZakazID, 
                OrderDept,
                OrderDate,
                StoreID,
                product_id, 
                price, 
                line_count, 
                line_num, 
                nvl(prod_volume,0), 
                nvl(prod_weight,0), 
                RequiredReservationFlag);

          if (valErrorMessage <> '') then raise productnotfound; end if;

        end loop;

   exception 
      when exLineNumber then
           rollback;
           valErrorMessage := 'no line number';
           HTP.print(GetErrorMessage('param_error'));
           HTP.print('</response>');
           return;
      when exProductID then
           rollback;
           valErrorMessage := 'no product id';
           HTP.print(GetErrorMessage('param_error'));
           HTP.print('</response>');
           return;
      when exLineCount then
           rollback;
           valErrorMessage := 'no quantity';
           HTP.print(GetErrorMessage('param_error'));
           HTP.print('</response>');
           return;
      when exPrice then
           rollback;
           valErrorMessage := 'no price';
           HTP.print(GetErrorMessage('param_error'));
           HTP.print('</response>');
           return;
      when productnotfound then
           rollback;
           HTP.print(GetErrorMessage('server_error'));
           HTP.print('</response>');
           return;
      when OTHERS THEN
           rollback;
           valErrorMessage := SUBSTR(SQLERRM, 1, 800);
           HTP.print(GetErrorMessage('other'));
           HTP.print('</response>');
           return;
  end;

  commit work;
  HTP.print(GetOkMessage('CreateOrder',order_id));
  HTP.print('</response>');

END reserve;



procedure SetZakazState8 (ZakazID in raw, StoreID in raw, xmldoc in clob, RRFlag in number) is

  order_id      varchar2(20);

  line_num	number(5,0);
  product_id	varchar2(20);
  line_count	number(15,0);
  price		number(15,2);
  point_from	varchar2(20);
  point_to	varchar2(20);
  prod_volume 	number(15,6);
  prod_weight 	number(15,6);

  ClientID	raw(8);
  OrderDept     raw(8); 
  OrderDate     date;


  productnotfound EXCEPTION;
  exProductID     EXCEPTION;
  exLineNumber    EXCEPTION;
  exLineCount     EXCEPTION;
  exPrice         EXCEPTION;
  exDelWay	  EXCEPTION;
  exDelStore	  EXCEPTION;
  exDelDate	  EXCEPTION;

begin
  --  ,    
  begin

     select DEPARTMENT, DOC_DATE, CODE  into OrderDept, OrderDate, order_id from DB1_DOCUMENT where ID = ZakazID;

     delete from DB1_RESERV where doc = ZakazID;
     delete from DB1_RESERVE where document = ZakazID;

     delete from DB1_LINE where document = ZakazID;
     delete from DB1_LINE where document in (Select d.ID from DB1_DOCUMENT d where d.PID = ZakazID and d.TYPE in(15859716,15859719));
     delete from DB1_DOCUMENT where PID = ZakazID and TYPE in(15859716,15859719);

     for C1 in (
         select
            extractvalue(VALUE(mytbl), '/product/@line')   as line_num,
            extractvalue(VALUE(mytbl), '/product/@id')     as product_id,
            extractvalue(VALUE(mytbl), '/product/@count')  as line_count,
            extractvalue(VALUE(mytbl), '/product/@price')  as price,
            extractvalue(VALUE(mytbl), '/product/@weight') as weight,
            extractvalue(VALUE(mytbl), '/product/@volume') as volume
         from table(XMLSequence(XMLType(xmldoc).extract('/order/products/product'))) mytbl)  
     loop

         line_num    := to_number(C1.line_num);
         product_id  := C1.product_id;
         line_count  := to_number(C1.line_count);
         price       := to_number(C1.price, '999999999999999.99');
         prod_weight := to_number(C1.weight,'999999999999999.999999');
         prod_volume := to_number(C1.volume,'999999999999999.999999');

         if (line_num is null)   then raise exLineNumber; end if;
         if (product_id is null) then raise exProductID; end if;
         if (line_count is null) then raise exLineCount; end if;
         if (price is null)      then raise exPrice; end if;

         valErrorMessage := '';
         CreateLine(
               ZakazID, 
               OrderDept,
               OrderDate,
               StoreID,
               product_id, 
               price,
               line_count,
               line_num,
               prod_volume,
               prod_weight,
               RRFlag);
         if (valErrorMessage <> '') then raise productnotfound; end if;

     end loop;
		
  EXCEPTION
     when exLineNumber then
          rollback;
          valErrorMessage := 'no line number';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when exProductID then
          rollback;
          valErrorMessage := 'no product id';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when exLineCount then
          rollback;
          valErrorMessage := 'no quantity';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when exPrice then
          rollback;
          valErrorMessage := 'no price';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when productnotfound then
          rollback;
          HTP.print(GetErrorMessage('server_error'));
          HTP.print('</response>');
          return;
     when OTHERS THEN
          rollback;
          valErrorMessage := SUBSTR(SQLERRM, 1, 800);
          HTP.print(GetErrorMessage('other'));
          HTP.print('</response>');
          return;
  end;

  update DB1_LINE set STATE = 1 where document = ZakazID;
  update DB1_DOCUMENT
     set F14352406 = HEXTORAW('00110003001E0006'),
         F63766532 = HEXTORAW('03CD000B03CD0001'),
         STATE =1
     where id = ZakazID;


  insert into ZAKAZ_STATE (ZAKAZ, STATE, CHANGE_DATE, USERID, SENDED_IM) 
  values (ZakazID, HEXTORAW('00110003001E0006'), SYSDATE, DOMINO.GetUserID(),1);

  commit work;

  HTP.print(GetOkMessage('ApproveOrder',order_id));
  HTP.print('</response>');

end SetZakazState8;



PROCEDURE approve(data in clob) is
  pragma AUTONOMOUS_TRANSACTION;
  RequiredReservationFlag number(5,0) := 1; 

  xmldoc         clob;
  order_id       varchar2(20);
  logistic_id    varchar2(20);
  storage_group  varchar2(20);

  DeliveryWay    varchar2(20);

  endpoint       varchar2(20);
  endpoint_from  varchar2(20);
  endpoint_to    varchar2(20);

  fio            varchar2(80);
  address        varchar2(120);
  region         varchar2(40);
  city           varchar2(120);
  street         varchar2(40);
  streetabbr     varchar2(40);
  house	         varchar2(40);
  housekorp      varchar2(40);
  housestr       varchar2(40);
  flat           varchar2(40);
  phone          varchar2(40);
  email          varchar2(40);
  Comments       varchar2(2000);

  deliverydate   varchar2(20);
  deliveryperiod varchar2(20);
  shippingdate   varchar2(20);
  paymentType    varchar2(20);

  line_num       number(5,0);
  product_id     varchar2(20);
  line_count     number(15,0);
  price	         number(15,2);
  prod_volume    number(15,6);
  prod_weight    number(15,6);
  ColLine        number(19,0);

  StoreID        raw(8);
  LogisticID     raw(8);
  EndpointID     raw(8);
  ClientID       raw(8);
  ZakazID        raw(8);
  OrderState     raw(8);
  OrderWay       raw(8);

  OrderIsChanged varchar2(3);

  productnotfound EXCEPTION;
  exProductID     EXCEPTION;
  exLineNumber    EXCEPTION;
  exLineCount     EXCEPTION;
  exPrice         EXCEPTION;
  exDelWay	  EXCEPTION;
  exDelStore	  EXCEPTION;
  exDelDate	  EXCEPTION;
  exRegion	  EXCEPTION;

begin
  xmldoc := convert(data,'CL8MSWIN1251','UTF8');
  writelog('<approve>' || Substr(to_char(xmldoc),1,1980));

  OWA_UTIL.mime_header('text/xml');
  HTP.print('<?xml version="1.0" encoding="utf-8"?>');
  HTP.print('<response>');


  --   
  begin
     select 
        extractvalue(VALUE(mytbl), '/order/@order_id'),
        extractvalue(VALUE(mytbl), '/order/@storage_group') 
     into order_id, storage_group
     from table(XMLSequence(XMLType(xmldoc).extract('/order'))) mytbl;
  exception when OTHERS then
     valErrorMessage := 'no order id';
     HTP.print(GetErrorMessage('param_error'));
     HTP.print('</response>');
     return;
  end;

  --   
  ZakazID := GetOrder(order_id,OrderState,OrderWay);
  if (ZakazID is null) then
      valErrorMessage := 'order not found order_id = ' || order_id;
      HTP.print(GetErrorMessage('server_error'));
      HTP.print('</response>');
      return;
  end if;

  --   
  begin
     select
        extractvalue(VALUE(mytbl), '/delivery/@way'),
        extractvalue(VALUE(mytbl), '/delivery/@endpoint'),
        extractvalue(VALUE(mytbl), '/delivery/@endpoint_from'),
        extractvalue(VALUE(mytbl), '/delivery/@endpoint_to'),
        extractvalue(VALUE(mytbl), '/delivery/@fio'),
        extractvalue(VALUE(mytbl), '/delivery/@address'),
        extractvalue(VALUE(mytbl), '/delivery/@region'),
        extractvalue(VALUE(mytbl), '/delivery/@city'),
        extractvalue(VALUE(mytbl), '/delivery/@street'),
        extractvalue(VALUE(mytbl), '/delivery/@streetAbbr'),
        extractvalue(VALUE(mytbl), '/delivery/@house'),
        extractvalue(VALUE(mytbl), '/delivery/@houseKorpus'),
        extractvalue(VALUE(mytbl), '/delivery/@str'),
        extractvalue(VALUE(mytbl), '/delivery/@flat'),
        extractvalue(VALUE(mytbl), '/delivery/@phone'),
        extractvalue(VALUE(mytbl), '/delivery/@remark'),
        extractvalue(VALUE(mytbl), '/delivery/@email'),
        extractvalue(VALUE(mytbl), '/delivery/@deliverydate'),
        extractvalue(VALUE(mytbl), '/delivery/@delivery_period'),
        extractvalue(VALUE(mytbl), '/delivery/@shippingdate'),
        extractvalue(VALUE(mytbl), '/delivery/@payment_method')
     into 
        DeliveryWay, 
        endpoint, 
        endpoint_from, 
        endpoint_to, 
        fio, 
        address, 
        region, 
        city, 
        street, 
        streetabbr, 
        house, 
        housekorp, 
        housestr, 
        flat, 
        phone, 
        Comments, 
        email, 
        deliverydate, 
        deliveryPeriod,
        shippingdate,
        paymentType
     from table(XMLSequence(XMLType(xmldoc).extract('/order/delivery'))) mytbl;

     if (DeliveryWay is null)  then raise exDelWay; end if;
     if (Endpoint is null)     then raise exDelStore; end if;
     if (deliverydate is null) then raise exDelDate; end if;
     if (region is null)       then raise exRegion; end if;

  exception 
     when exDelWay then
          valErrorMessage := 'no delivery way';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when exDelStore then
          valErrorMessage := 'no store id';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when exDelDate then
          valErrorMessage := 'no delivery date';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when exRegion then
          valErrorMessage := 'no region';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when OtHERS then
          valErrorMessage := 'no delivery parameters';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
  end;

  --   
  begin
     select extractvalue(VALUE(mytbl), '/logistic/@id') into logistic_id
     from table(XMLSequence(XMLType(xmldoc).extract('/order/logistic'))) mytbl;
  exception when OTHERS then
     null;
  end;

  --    
  StoreID := GetStore(endpoint);
  if (StoreID is null) then  --     /
      valErrorMessage := 'bad store id = ' || endpoint;
      HTP.print(GetErrorMessage('param_error'));
      HTP.print('</response>');
      return;
  end if;

  --    
  --   :
  --      ""      (..      WAY)

  if (DeliveryWay <> WayTypeDescription(OrderWay)) then

      if (OrderState <> HEXTORAW('00110003001E0001')) then  --       
          valErrorMessage := 'wrong delivery, order must be revoked';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
      end if;

  end if;


  case DeliveryWay
     when 'logist' then
          --      
          case OrderState
             when HEXTORAW('00110003001E0001') then null; --  ()             
             when HEXTORAW('0011000301B50002') then null; --      (  )    
             when HEXTORAW('00110003001E0006') then null; --                         
             when HEXTORAW('0011000303CD0001') then null; --                            
             when HEXTORAW('0011000303CD0003') then null; --                    
             when HEXTORAW('0011000303CD0004') then null; --                   
          else 
          --      - 
              valErrorMessage := 'the order ' || order_id || ' not valid state';
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end case;

          LogisticID := GetLogistic(logistic_id);
          if (LogisticID is null) then
          --       
              valErrorMessage := 'bad logistic id = ' || logistic_id;
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

     when 'pickup' then
          --      
          if (OrderState <> HEXTORAW('00110003001E0001')) then  --       
              valErrorMessage := 'the order ' || order_id || ' has already been approved';
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

          LogisticID := GetLogistic(logistic_id);
   	  if (LogisticID is null) then             --       
              valErrorMessage := 'bad logistic id = ' || logistic_id;
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

          EndpointID := GetStore(endpoint_to);
          if (EndpointID is null) then             --       
              valErrorMessage := 'bad salespoint id = ' || endpoint_to;
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

          if (paymentType is null) then 
              valErrorMessage := 'payment_method not defined';
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

          if (paymentType <> 'cash') then 
              valErrorMessage := 'payment_method not allowed';
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

     when 'instore' then
          RequiredReservationFlag := 0;                         --   

          --      
          if (OrderState <> HEXTORAW('00110003001E0001')) then  --       
              valErrorMessage := 'the order ' || order_id || ' has already been approved';
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

          EndpointID := GetStore(endpoint_to);
          if (EndpointID is null) then             --       
              valErrorMessage := 'bad salespoint id = ' || endpoint_to;
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

     when 'courier' then null;

     else --      
          valErrorMessage := 'bad delivery way type = ' || DeliveryWay;
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
  end case;


  if (phone is not null) then
      ClientID := GetClient(phone);
  end if;


  if (ClientID is null) then

      CreateClient(fio,phone,email,address,region, city, street, streetabbr, house, housekorp, housestr, flat);

      ClientID := valClientID;

      if (ClientID is null) then
          valErrorMessage := 'error create client';
          HTP.print(GetErrorMessage('server_error'));
          HTP.print('</response>');
          return;
      end if;
  else
      UpdateClient(ClientID,fio,email,address,region, city, street, streetabbr, house, housekorp, housestr, flat);
  end if;


  --        
  UpdateOrder(
        ZakazID, 
        StoreID, 
        ClientID,
        DeliveryWay,
        to_date(deliverydate,'dd/mm/yyyy'),
        DeliveryPeriod,
        to_date(ShippingDate,'dd/mm/yyyy'),
        PaymentType, 
        Comments,
        endpoint_to,
        EndpointID,
        LogisticID,
        address,
        region,
        storage_group);

  --   ""
  SetZakazState8(ZakazID, StoreID, xmldoc, RequiredReservationFlag);

END approve;



PROCEDURE change(data in clob) is

  pragma AUTONOMOUS_TRANSACTION;

  xmldoc         clob;
  order_id       varchar2(20);
  storage_group  varchar2(20);
  logistic_id    varchar2(20);
  endpoint       varchar2(20);
  endpoint_to    varchar2(20);

  fio            varchar2(80);
  phone          varchar2(40);
  email          varchar2(40);
  address        varchar2(120);
  region	 varchar2(40);
  city		 varchar2(120);
  street	 varchar2(40);
  streetabbr	 varchar2(40);
  house		 varchar2(40);
  housekorp	 varchar2(40);
  housestr	 varchar2(40);
  flat		 varchar2(40);
  Comments       varchar2(2000);
  DeliveryWay    varchar2(20);
  CurrentWay     varchar2(20);

  deliverydate   varchar2(20);
  deliveryPeriod varchar2(20);
  shippingdate   varchar2(20);
  paymentType    varchar2(20);

  product_id	 varchar2(20);
  price		 number(15,2);
  point_from	 varchar2(20);
  point_to	 varchar2(20);
  prod_volume 	 number(15,6);
  prod_weight 	 number(15,6);
  ColLine	 number(19,0);
  line_num	 number(5,0);
  line_count	 number(15,0);

  StoreID	  raw(8);
  LogisticID	  raw(8);
  EndpointID      raw(8);
  ClientID	  raw(8);
  ZakazID	  raw(8);
  OrderState      raw(8);
  OrderWay        raw(8);
  OrderStateCode  varchar2(3);
  OrderLogistic   raw(8);

  productnotfound EXCEPTION;
  exProductID     EXCEPTION;
  exLineNumber    EXCEPTION;
  exLineCount     EXCEPTION;
  exPrice         EXCEPTION;
  exDelWay	  EXCEPTION;
  exDelStore	  EXCEPTION;
  exDelDate	  EXCEPTION;

begin

  xmldoc := convert(data,'CL8MSWIN1251','UTF8');
  writelog('<change>' || Substr(to_char(xmldoc),1,1980));

  OWA_UTIL.mime_header('text/xml');
  HTP.print('<?xml version="1.0" encoding="utf-8"?>');
  HTP.print('<response>');

  --   
  begin
     select
        extractvalue(VALUE(mytbl), '/order/@order_id'),
        extractvalue(VALUE(mytbl), '/order/@storage_group') 
     into order_id, storage_group
     from table(XMLSequence(XMLType(xmldoc).extract('/order'))) mytbl;
  exception when OTHERS then
     valErrorMessage := 'no order id';
     HTP.print(GetErrorMessage('param_error'));
     HTP.print('</response>');
     return;
  end;

  --   
  ZakazID := GetOrder(order_id,OrderState,OrderWay);
  if (ZakazID is null) then               --     
      valErrorMessage := 'order not found order_id = ' || order_id;
      HTP.print(GetErrorMessage('server_error'));
      HTP.print('</response>');
      return;
  end if;

  --   
  begin
     select
        extractvalue(VALUE(mytbl), '/delivery/@way'),
        extractvalue(VALUE(mytbl), '/delivery/@endpoint'),
        extractvalue(VALUE(mytbl), '/delivery/@endpoint_to'),
        extractvalue(VALUE(mytbl), '/delivery/@fio'),
        extractvalue(VALUE(mytbl), '/delivery/@address'),
        extractvalue(VALUE(mytbl), '/delivery/@region'),
        extractvalue(VALUE(mytbl), '/delivery/@city'),
        extractvalue(VALUE(mytbl), '/delivery/@street'),
        extractvalue(VALUE(mytbl), '/delivery/@streetAbbr'),
        extractvalue(VALUE(mytbl), '/delivery/@house'),
        extractvalue(VALUE(mytbl), '/delivery/@houseKorpus'),
        extractvalue(VALUE(mytbl), '/delivery/@str'),
        extractvalue(VALUE(mytbl), '/delivery/@flat'),
        extractvalue(VALUE(mytbl), '/delivery/@phone'),
        extractvalue(VALUE(mytbl), '/delivery/@remark'),
        extractvalue(VALUE(mytbl), '/delivery/@email'),
        extractvalue(VALUE(mytbl), '/delivery/@deliverydate'),
        extractvalue(VALUE(mytbl), '/delivery/@delivery_period'),
        extractvalue(VALUE(mytbl), '/delivery/@shippingdate'),
        extractvalue(VALUE(mytbl), '/delivery/@payment_method')
     into 
        DeliveryWay, 
        endpoint, 
        endpoint_to, 
        fio,
        address,
        region,
        city,
        street,
        streetabbr,
        house,
        housekorp,
        housestr,
        flat, 
        phone, 
        Comments, 
        email, 
        deliverydate, 
        deliveryPeriod, 
        shippingdate, 
        paymentType
     from table(XMLSequence(XMLType(xmldoc).extract('/order/delivery'))) mytbl;

     if (DeliveryWay is null)  then raise exDelWay;   end if;
     if (endpoint is null)     then raise exDelStore; end if;
     if (deliverydate is null) then raise exDelDate;  end if;

  exception 
     when exDelWay then
          valErrorMessage := 'no delivery way';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when exDelStore then
          valErrorMessage := 'no store id';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when exDelDate then
          valErrorMessage := 'no delivery date';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
     when OtHERS then
          valErrorMessage := 'no delivery parameters';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
  end;


  --   
  begin
     select extractvalue(VALUE(mytbl), '/logistic/@id') into logistic_id
     from table(XMLSequence(XMLType(xmldoc).extract('/order/logistic'))) mytbl;
  exception when OTHERS then
     valErrorMessage := 'no logistic id';
     HTP.print(GetErrorMessage('param_error'));
     HTP.print('</response>');
     return;
  end;

  --    
  StoreID := GetStore(endpoint);
  if (StoreID is null) then      --     /
      valErrorMessage := 'bad store id = ' || endpoint;
      HTP.print(GetErrorMessage('param_error'));
      HTP.print('</response>');
      return;
  end if;

  --          ""      
  -- ..      WAY
 
  if (DeliveryWay <> WayTypeDescription(OrderWay)) then

      if (OrderState <> HEXTORAW('00110003001E0001')) then  --       
          valErrorMessage := 'wrong delivery, order must be revoked';
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
      end if;

      if (DeliveryWay = 'instore') then 
          begin                    --  
            delete from DB1_RESERV where doc = ZakazID;
            delete from DB1_RESERVE where document = ZakazID;
            commit;
          end;
      end if;

  end if;


  case DeliveryWay
     when 'logist' then
          --      
          case OrderState
             when HEXTORAW('00110003001E0001') then null; --  ()             
             when HEXTORAW('0011000301B50002') then null; --      (  )    
             when HEXTORAW('00110003001E0006') then null; --                         
             when HEXTORAW('0011000303CD0001') then null; --                            
             when HEXTORAW('0011000303CD0003') then null; --                    
             when HEXTORAW('0011000303CD0004') then null; --                   
          else 
          --      - 
              valErrorMessage := 'the order ' || order_id || ' not valid state';
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end case;

          LogisticID := GetLogistic(logistic_id);
          if (LogisticID is null) then
          --       
              valErrorMessage := 'bad logistic id = ' || logistic_id;
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

     when 'pickup' then
          --      
          case OrderState
             when HEXTORAW('00110003001E0001') then null; --  ()             
             when HEXTORAW('0011000301B50002') then null; --      (  )    
             when HEXTORAW('00110003001E0006') then null; --                         
             when HEXTORAW('0011000303CD0001') then null; --                            
             when HEXTORAW('0011000303CD0003') then null; --                    
             when HEXTORAW('0011000303CD0004') then null; --                   
          else 
          --      - 
              valErrorMessage := 'the order ' || order_id || ' not valid state';
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end case;

          LogisticID := GetLogistic(logistic_id);
   	  if (LogisticID is null) then             --       
              valErrorMessage := 'bad logistic id = ' || logistic_id;
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

          --         
          --       endpoint_to, EndpointID, LogisticID,
          --   ,  ,   ,      .             

          select F33620503 into OrderLogistic from DB1_DOCUMENT where CLASS=14352386 and TYPE=1966128 and ID=ZakazID;

          if (OrderLogistic <> LogisticID) then  --    
              valErrorMessage := 'bad logistic id = ' || logistic_id;
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

          EndpointID := GetStore(endpoint_to);
          if (EndpointID is null) then             --       
              valErrorMessage := 'bad salespoint id = ' || endpoint_to;
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

          if (paymentType is null) then 
              valErrorMessage := 'payment_method not defined';
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

          if (paymentType <> 'cash') then 
              valErrorMessage := 'payment_method not allowed';
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

     when 'instore' then
          --      
          if (OrderState <> HEXTORAW('00110003001E0001')) then  --       
              valErrorMessage := 'the order ' || order_id || ' has already been approved';
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

          EndpointID := GetStore(endpoint_to);
          if (EndpointID is null) then             --       
              valErrorMessage := 'bad salespoint id = ' || endpoint_to;
              HTP.print(GetErrorMessage('param_error'));
              HTP.print('</response>');
              return;
          end if;

     when 'courier' then null;

     else --      
          valErrorMessage := 'bad delivery way type = ' || DeliveryWay;
          HTP.print(GetErrorMessage('param_error'));
          HTP.print('</response>');
          return;
  end case;


  if (phone is not null) then
      ClientID := GetClient(phone);
  end if;

  if (ClientID is null) then             -- create client

      CreateClient(fio,phone,email,address,region, city, street, streetabbr, house, housekorp, housestr, flat);

      ClientID := valClientID;

      if (ClientID is null) then     -- send error message
       	  valErrorMessage := 'error create client';
       	  HTP.print(GetErrorMessage('server_error'));
       	  HTP.print('</response>');
       	  return;
      end if;
  else
      UpdateClient(ClientID,fio,email,address,region, city, street, streetabbr, house, housekorp, housestr, flat);
  end if;

  UpdateOrder(
      ZakazID, 
      StoreID, 
      ClientID,
      DeliveryWay,
      to_date(deliverydate,'dd/mm/yyyy'),
      deliveryPeriod,
      to_date(shippingDate,'dd/mm/yyyy'),
      paymentType, 
      Comments,
      endpoint_to,
      EndpointID,
      LogisticID,
      address,
      region,
      storage_group);
  commit;

  HTP.print(GetOkMessage('ChangeOrder', order_id));
  HTP.print('</response>');

END change;



PROCEDURE revoke_(data in clob) is
  pragma AUTONOMOUS_TRANSACTION;
  xmldoc     clob;
  order_id   varchar2(20);
  reason     varchar2(120);
  OrderID    raw(8);
  OrderState raw(8);
  OrderWay   raw(8);
begin

  xmldoc := convert(data,'CL8MSWIN1251','UTF8');
  writelog('<revoke_>' || Substr(to_char(xmldoc),1,1980));

  OWA_UTIL.mime_header('text/xml');
  HTP.print('<?xml version="1.0" encoding="utf-8"?>');
  HTP.print('<response>');

  --   
  begin
     select
        extractvalue(VALUE(mytbl), '/order/@order_id'),
        extractvalue(VALUE(mytbl), '/order/@reason')
     into order_id, reason
     from table(XMLSequence(XMLType(xmldoc).extract('/order'))) mytbl;
  exception when OTHERS then
     valErrorMessage := 'no order id';
     PrintErrorMessage('param_error'); goto Ext;
  end;

  --   
  OrderID := GetOrder(order_id,OrderState,OrderWay);
  if (OrderID is null) then
      valErrorMessage := 'order not found order_id = ' || order_id;
      PrintErrorMessage('server_error'); goto Ext;
  end if;

  --    
  case OrderState
      when HEXTORAW('00110003001E0001') then null; --  ()             
      when HEXTORAW('0011000301B50002') then null; --      (  )    
      when HEXTORAW('00110003001E0006') then null; --                         
      when HEXTORAW('0011000303CD0001') then null; --                            
      when HEXTORAW('0011000303CD0003') then null; --                    
      when HEXTORAW('0011000303CD0004') then null; --                   
      when HEXTORAW('0011000303CD0002') then null; --  ()
  else 
  --      - 
      valErrorMessage := 'order ' || order_id || ' cannot be revoked';
      PrintErrorMessage('server_error'); goto Ext;
  end case;

  --  
  RevokeOrder(OrderID, reason);
  HTP.print(GetOkMessage('RevokeOrder',order_id));

<<Ext>>
  HTP.print('</response>');
END revoke_;



PROCEDURE getorderstate(data in clob) is
  xmldoc       clob;
  order_id     varchar2(20);
  OrderID      raw(8);
  OrderState   raw(8);
  OrderWay     raw(8);
begin

  xmldoc := convert(data,'CL8MSWIN1251','UTF8');
  writelog('<getorderstate>' || Substr(to_char(xmldoc),1,1980));

  OWA_UTIL.mime_header('text/xml');
  HTP.print('<?xml version="1.0" encoding="utf-8"?>');
  HTP.print('<response>');

  --   
  begin
     select extractvalue(VALUE(mytbl), '/order/@order_id') into order_id
     from table(XMLSequence(XMLType(xmldoc).extract('/order'))) mytbl;
  exception when OTHERS then
     valErrorMessage := 'no order id';
     PrintErrorMessage('param_error'); goto Ext;
  end;

  --    
  OrderID := GetOrder(order_id,OrderState,OrderWay);

  if (OrderID is null) then
      valErrorMessage := 'order not found order_id = ' || order_id;
      PrintErrorMessage('server_error'); goto Ext;
  end if;

  HTP.print(GetOkMessage('GetOrderState',order_id));

<<Ext>>
  HTP.print('</response>');
END getorderstate;                                                   



PROCEDURE presence(data in clob) is

  xmldoc        clob;
  RegionCode    varchar2(40);
  RegionID      raw(8);
  StoreID       raw(8);
  ProductCode   varchar2(40);
  ProductID     raw(8);
  ProductStock  number(19,3);
  isPresent     varchar2(10);

begin

  xmldoc := convert(data,'CL8MSWIN1251','UTF8');

  OWA_UTIL.mime_header('text/xml');
  HTP.print('<?xml version="1.0" encoding="utf-8"?>');
  HTP.print('<response>');

  --   
  begin
      select extractvalue(VALUE(mytbl), '/region/@region_id') into RegionCode
      from table(XMLSequence(XMLType(xmldoc).extract('/region'))) mytbl;
  exception when OTHERS then
      valErrorMessage := 'no region id';
      PrintErrorMessage('param_error'); goto Ext;
  end;

  --  UID   
  RegionID := GetRegion(RegionCode);
  if (RegionID is null) then
      valErrorMessage := 'bad region id = ' || RegionCode;
      PrintErrorMessage('server_error'); goto Ext;
  end if;

  --  
  HTP.print('<meta code="200"/>');
  HTP.print('<body>');        
  for C1 in (
      Select id, F15073296 as code 
         from DB1_AGENT 
         where pid = RegionID and type = 14745607 and F15073296 is not null) 
      loop

         StoreId := C1.id;
         HTP.print('<store store_id="' || C1.code || '">');
         HTP.print('<products>');

         for C2 in (
             Select
                extractvalue(VALUE(mytbl), '/product/@line')  as line_num,
                extractvalue(VALUE(mytbl), '/product/@id')    as product_id,
                extractvalue(VALUE(mytbl), '/product/@count') as line_count
             from table(XMLSequence(XMLType(xmldoc).extract('/region/products/product'))) mytbl)  
             loop

                ProductCode  := C2.product_id;
                ProductStock := 0;

                begin
                   select id into ProductID
                   from DB1_PRODUCT
                   where CLASS=14745601 and CODE=ProductCode;
                exception when OTHERS then 
                   ProductID := NULL;
                end;

                if (ProductID is not null) then
                    Select NVL(Sum(t1.qty),0) into ProductStock
                       from (
                           Select qty 
                              from DB1_TRANSFER_A2_VIEW  
                              where (DEPT = StoreID) and (PRODUCT = ProductID)
                           union all
                           Select Sum(-amount) as qty 
                              from DB1_VRSTOCK 
                              where (DEPT = StoreID) and (PRODUCT = ProductID)
                       ) t1;
                end if;

                if (C2.line_count > ProductStock) then
                    isPresent := 'false';
                else
                    isPresent := 'true';
                end if;

                HTP.print('<product line="' || C2.line_num || '" id="' || ProductCode ||'" presence_count="' || ProductStock || '" is_present="' || isPresent || '"/>');

              end loop;

	 HTP.print('</products>');
	 HTP.print('</store>');
     end loop;

   HTP.print('</body>');        	

<<Ext>>
   HTP.print('</response>');
EXCEPTION WHEN OTHERS THEN
   HTP.print('</response>');
END presence;



PROCEDURE price(data in clob) is
  xmldoc        clob;
  ProductCode   varchar2(40);
  ProductID     raw(8);
  ProductPrice  varchar2(40);
  RegionCode    varchar2(40);
  RegionID      raw(8);

begin

  xmldoc := convert(data,'CL8MSWIN1251','UTF8');
  writelog('<price>' || Substr(to_char(xmldoc),1,1980));

  OWA_UTIL.mime_header('text/xml');
  HTP.print('<?xml version="1.0" encoding="utf-8"?>');
  HTP.print('<response>');
  HTP.print('<meta code="200"/>');
  HTP.print(' <body>');        
  HTP.print('  <prices>');

  for C2 in (
      Select
         extractvalue(VALUE(mytbl), '/product/@id')     as product_id,
         extractvalue(VALUE(mytbl), '/product/@region') as region_id
      from table(XMLSequence(XMLType(xmldoc).extract('/prices/product'))) mytbl)  
  loop

      ProductCode  := C2.product_id;
      RegionCode   := C2.region_id;
      ProductPrice := 0;
      RegionID     := GetRegion(RegionCode);

      begin
         select id into ProductID from DB1_PRODUCT where CLASS in (14745601,3342339) and CODE=ProductCode;
      exception when OTHERS then 
         ProductID := NULL;
      end;


      if (ProductID is not null) then
          Select substr(max(to_char(trans_date,'yyyymmddhh24miss')||to_char(amount)),15,19)
          into ProductPrice
          from db1_trans_15073281
          where FINANCE_DEPT = HEXTORAW('9000000000000001')
            and DEBT_ACCOUNT = HEXTORAW('0000008300660002')
            and DEBT_A1 = ProductID
            and DEBT_A2 = RegionID
            and DEBT_A3 = HEXTORAW('0000016700E50002')
            and CURRENCY = HEXTORAW('07D2000901AD0001');
      end if;

      HTP.print('   <product id="' || ProductCode || '">' || ProductPrice || '</product>');

  end loop;

  HTP.print('  </prices>');
  HTP.print(' </body>');        	
  HTP.print('</response>');

EXCEPTION WHEN OTHERS THEN
  HTP.print('<meta code="400" error_type=""/>');
  HTP.print('</response>');
  return;

END price;



------------------------------------------------------------------------------------------------
--
--    
--
------------------------------------------------------------------------------------------------

FUNCTION GetRegion(RegionCode in varchar2) return raw is
   RegionID DB1_AGENT.ID%TYPE;
begin
   begin
      select id into RegionID
      from DB1_AGENT
      where CLASS=2 and TYPE in (14745602,15073286,40566785,40566786) and F15073296=RegionCode;
   exception
      when OTHERS then RegionID := NULL;
   end;
   return RegionID;
end;


FUNCTION GetStore(StoreCode in varchar2) return raw is
   StoreID DB1_AGENT.ID%TYPE;
begin
   begin
      select id into StoreID
      from DB1_AGENT
      where CLASS=2 and TYPE=14745607 and F15073296=StoreCode;
   exception
      when OTHERS then StoreID := NULL;
   end;
   return StoreID;
end;


FUNCTION GetClient(ClientCode in varchar2) return raw is
   ClientID DB1_AGENT.ID%TYPE;
begin
   begin
      select id into ClientID
      from DB1_AGENT
      where CLASS=14745602 and TYPE=63766529 and CODE=ClientCode;
   exception
      when OTHERS then ClientID := NULL;
   end;
   return ClientID;
end;


FUNCTION GetLogistic(ClientCode in varchar2) return raw is
   ClientID DB1_AGENT.ID%TYPE;
begin
   begin
      select id into ClientID
      from DB1_AGENT
      where CLASS=14745601 and TYPE=14745610 and CODE=ClientCode;
   exception
      when OTHERS then ClientID := NULL;
   end;
   return ClientID;
end;


FUNCTION GetOrder(OrderCode in varchar2, OrderState out raw, OrderWay out raw) return raw is
   ZakazID DB1_DOCUMENT.ID%TYPE;
begin
   begin
      select ID, F14352406, F28835862
      into ZakazID, OrderState, OrderWay
      from DB1_DOCUMENT
      where CLASS=14352386 and TYPE=1966128 and CODE=OrderCode;
   exception
      when OTHERS then ZakazID := NULL;
   end;
   return ZakazID;
end;


FUNCTION WayTypeDescription(Delivery in raw) return varchar2 is
begin 
   case Delivery
      when HEXTORAW('01B8001801B80002') then return 'logist'; 
      when HEXTORAW('01B8001801B80001') then return 'pickup'; 
      when HEXTORAW('01B8001801B80003') then return 'courier';
      when HEXTORAW('01B8001801B80004') then return 'instore';
      else return RAWTOHEX(Delivery);
   end case;
exception
   when OTHERS then return null;
end;


FUNCTION DecodeOrderState(OrderState in raw) return varchar2 is
begin
   --      (     )
   case OrderState
      when HEXTORAW('00110003001E0001') then return '0';    --   ()                          
      when HEXTORAW('0011000301B50002') then return '9';    --   (  )                     
      when HEXTORAW('00110003001E0006') then return '8';    --                                       
      when HEXTORAW('0011000303CD0001') then return '9';    --                                          
      when HEXTORAW('0011000303CD0003') then return '5';    --                                  
      when HEXTORAW('0011000303CD0004') then return '8';    --                                 
      when HEXTORAW('0011000301C60002') then return '4';    --                                       
      when HEXTORAW('0011000303CD0002') then return '4';    --  ()                            
      when HEXTORAW('0011000301C60006') then return '6';    --                                        
      when HEXTORAW('0011000302870001') then return '10';   --                                    
      when HEXTORAW('0011000302870003') then return '11';   --   
      when HEXTORAW('0011000301B50005') then return '1';    --   ()                       
      when HEXTORAW('0011000303CD0005') then return '2';    --    ( )  
      when HEXTORAW('0011000303CD0007') then return '3';    --                                
      when HEXTORAW('0011000301C60005') then return '7';    --                                     
      else return '-1';
   end case;
exception
   when OTHERS then return '-1';
end;


PROCEDURE PrintErrorMessage(text in varchar2) is
begin
  HTP.print('<meta code="400" error_type="' || text || '" error_message="' || valErrorMessage || '"/>');
end;


FUNCTION GetErrorMessage(Error_Type in varchar2) return varchar2 is
begin
  return '<meta code="400" error_type="' || Error_Type || '" error_message="' || valErrorMessage || '"/>';
end;


FUNCTION GetOkMessage(
   typeMessage in varchar2, 
   order_id    in varchar2) return varchar2 
is
   out_message varchar2(20000);
   zero_count  number(4,0);
   OrderID     raw(8);
   OrderState  raw(8);
   OrderWay    raw(8);
   StateCode   varchar2(4);
begin
   OrderID := GetOrder(order_id,OrderState,OrderWay);

   StateCode := DecodeOrderState(OrderState);

   case typeMessage
      when 'CreateOrder' then
         begin
            select count(*) into zero_count  from db1_line l, db1_product p
            where l.document = OrderID and p.id = l.product and l.F14286852=0 and p.type = 14745601;
         exception when OTHERS then
            zero_count := 0;
         end;
         if zero_count = 0 then
            out_message := '<meta code="200"/>';
         else               
            out_message := '<meta code="201"/>';
         end if;

      when 'ApproveOrder' then
         out_message := '<meta code="200"/>';

      when 'RevokeOrder' then
         out_message := '<meta code="200"/>';
         out_message := out_message || '<order order_id="' || order_id || '"/>';
         return out_message;

      when 'GetOrder' then
         out_message := '<meta code="200"/>';
         out_message := out_message || '<order order_id="' || order_id || '" state="' || StateCode || '"/>';

      when 'GetOrderState' then
         out_message := '<meta code="200"/>';
         out_message := out_message || '<order order_id="' || order_id || '" state="' || StateCode || '"/>';
         return out_message;

      when 'ChangeOrder' then
         out_message := '<meta code="200"/>';
         out_message := out_message || '<order order_id="' || order_id || '" state="' || StateCode || '"/>';
         return out_message;
  end case;

  out_message := out_message || '<body>';  
  out_message := out_message || ' <order order_id="' || order_id || '">';
  out_message := out_message || '  <products>';

  for c1 in (
      select l.line_no as line, p.code as id, l.F14286852 as reserved_count
      from db1_line l, db1_product p
      where l.document = OrderID and p.id = l.product) 
  loop
      out_message := out_message || '   <product line="' || c1.line || '" id="' || c1.id || '" reserved_count="' || c1.reserved_count ||'"/>';
  end loop;

  out_message := out_message || '  </products>';
  out_message := out_message || ' </order>';
  out_message := out_message || '</body>';    

  return out_message;
end;


PROCEDURE writelog(text in varchar2) is
begin
  insert into query_all (q_date, query_text)  values (sysdate, text);
  commit;
end;


END zakaz;

--    (   ,      )

/
SHOW ERRORS

exit;
